﻿
#ifndef __ZEGOEXPRESSBRIDGE_H__
#define __ZEGOEXPRESSBRIDGE_H__

#include <string>

#include "./include/zego-express-errcode.h"
#include "./include/zego-express-engine.h"
#include "./include/zego-express-room.h"
#include "./include/zego-express-publisher.h"
#include "./include/zego-express-player.h"
#include "./include/zego-express-device.h"
#include "./include/zego-express-preprocess.h"
#include "./include/zego-express-im.h"
#include "./include/zego-express-mixer.h"
#include "./include/zego-express-mediaplayer.h"
#include "./include/zego-express-record.h"
#include "./include/zego-express-custom-video-io.h"
#include "./include/zego-express-custom-audio-io.h"
#include "./include/zego-express-utils.h"


namespace ZEGO {
    namespace EXPRESS {

        class  ZegoExpressEngineBridge
        {
            class ZegoExpressEngineBridgePri {
            public:
                void init() {
                    this->debugVerboseLanguage = zego_language_english;
                    this->debugVerboseEnabled = true;
                    this->isTestEnv = false;
                }
                zego_language debugVerboseLanguage = zego_language_english;
                bool debugVerboseEnabled = true;
                bool isTestEnv = false;

                bool enableToastOnTestEnv = true;
                bool enableDebugErrorAlways = false;
            };
            
        public:
            static ZegoExpressEngineBridge *GetInstance()
            {
                static ZegoExpressEngineBridge oInstance;
                return &oInstance;
            }

            const char *getVersion()
            {
                return zego_express_get_version();
            }

            void setDebugVerbose(bool enable, zego_language language)
            {
                pri->debugVerboseEnabled = enable;
                pri->debugVerboseLanguage = language;
                zego_express_set_debug_verbose(enable, language);
            }

            void setAndroidEnv(void* jvm, void* ctx)
            {
                zego_express_set_android_env(jvm, ctx);
            }

            void setEngineConfig(zego_engine_config config)
            {
                std::string advanceConfig = config.advanced_config;

                // keep enableErrorMessageBox on if find no "enable_toast_on_test_env=false" here
                std::string::size_type loc = advanceConfig.find("enable_toast_on_test_env=false", 0);
                pri->enableToastOnTestEnv = (loc == std::string::npos);

                // enable debug error when no error occurred if find "enable_debug_error_always=true" here
                loc = advanceConfig.find("enable_debug_error_always=true", 0);
                pri->enableDebugErrorAlways = (loc != std::string::npos);

                zego_express_set_engine_config(config);
            }

            int init(unsigned int appID, const char *appSign, bool isTestEnvironment, zego_scenario scenario)
            {
                // reset the locals
                pri->init();

                // set platform
                zego_express_set_platform_language(zego_platform_language_cpp);

                // do init
                pri->isTestEnv = isTestEnvironment;
                int result = zego_express_engine_init(appID, appSign, isTestEnvironment, scenario);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_ENGINE, __FUNCTION__, result);
                return result;
            }

            void uninitAsync()
            {
                int result = zego_express_engine_uninit_async();
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_ENGINE, __FUNCTION__, result);
            }

            void uploadLog()
            {
                zego_express_upload_log();
            }

            void loginRoom(const char *room_id, zego_user user, zego_room_config *room_config)
            {
                int result = zego_express_login_room(room_id, user, room_config);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_ROOM, __FUNCTION__, result);
            }

            void logoutRoom(const char *room_id)
            {
                int result = zego_express_logout_room(room_id);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_ROOM, __FUNCTION__, result);
            }

            void setVideoMirrorMode(zego_video_mirror_mode mirrorMode, enum zego_publish_channel channel)
            {
                int result = zego_express_set_video_mirror_mode(mirrorMode, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void startPreview(zego_canvas *canvas, enum zego_publish_channel channel)
            {
                int result = zego_express_start_preview(canvas, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void stopPreview(enum zego_publish_channel channel)
            {
                int result = zego_express_stop_preview(channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void setAudioConfig(zego_audio_config audioConfig)
            {
                int result = zego_express_set_audio_config(audioConfig);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            zego_audio_config getAudioConfig()
            {
                return zego_express_get_audio_config();
            }

            void setVideoConfig(zego_video_config videoConfig, enum zego_publish_channel channel)
            {
                int result = zego_express_set_video_config(videoConfig, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            zego_video_config getVideoConfig(enum zego_publish_channel channel)
            {
                return zego_express_get_video_config(channel);
            }

            void startPublishingStream(const char *streamID, enum zego_publish_channel channel)
            {
                int result = zego_express_start_publishing_stream(streamID, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void stopPublishingStream(enum zego_publish_channel channel)
            {
                int result = zego_express_stop_publishing_stream(channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            int setStreamExtraInfo(const char *extraInfo, enum zego_publish_channel channel)
            {
                int seq = zego_express_set_publish_stream_extra_info(extraInfo, channel);
                return seq;
            }

            void mutePublishStreamAudio(bool mute, enum zego_publish_channel channel)
            {
                int result = zego_express_mute_publish_stream_audio(mute, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void mutePublishStreamVideo(bool mute, enum zego_publish_channel channel)
            {
                int result = zego_express_mute_publish_stream_video(mute, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void setCaptureVolume(int volume)
            {
                int result = zego_express_set_capture_volume(volume);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            int addPublishCdnUrl(const char *streamID, const char *targetURL)
            {
                int seq = zego_express_add_publish_cdn_url(streamID, targetURL);
                return seq;
            }

            int removePublishCdnUrl(const char *streamID, const char *targetURL)
            {
                int seq = zego_express_remove_publish_cdn_url(streamID, targetURL);
                return seq;
            }

            void enablePublishDirectToCDN(bool enable, zego_cdn_config *config, enum zego_publish_channel channel)
            {
                int result = zego_express_enable_publish_direct_to_cdn(enable, config, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void setPublishWatermark(zego_watermark *watermark, bool isPreviewVisible, enum zego_publish_channel channel)
            {
                int result = zego_express_set_publish_watermark(isPreviewVisible, watermark, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void enableHardwareEncoder(bool enable)
            {
                int result = zego_express_enable_hardware_encoder(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void setCapturePipelineScaleMode(zego_capture_pipeline_scale_mode mode)
            {
                int result = zego_express_set_capture_pipeline_scale_mode(mode);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void sendSEI(const unsigned char *buffer, unsigned int buffer_length, enum zego_publish_channel channel)
            {
                int result = zego_express_send_sei(buffer, buffer_length, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void enableTrafficControl(bool enable, int property)
            {
                int result = zego_express_enable_traffic_control(enable, property);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void setMinVideoBitrateForTrafficControl(int bitrate, enum zego_traffic_control_min_video_bitrate_mode mode)
            {
                int result = zego_express_set_min_video_bitrate_for_traffic_control(bitrate, mode);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void setAppOrientation(zego_orientation orientation, zego_publish_channel channel)
            {
                int result = zego_express_set_app_orientation(orientation, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void startPlayingStream(const char *streamID, zego_canvas *canvas)
            {
                int result = zego_express_start_playing_stream(streamID, canvas);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PLAYER, __FUNCTION__, result);
            }

            void startPlayingStreamWithConfig(const char *streamID, zego_canvas *canvas, zego_player_config config)
            {
                int result = zego_express_start_playing_stream_with_config(streamID, canvas, config);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PLAYER, __FUNCTION__, result);
            }

            void stopPlayingStream(const char *streamID)
            {
                int result = zego_express_stop_playing_stream(streamID);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PLAYER, __FUNCTION__, result);
            }

            void setPlayVolume(const char *streamID, int volume)
            {
                int result = zego_express_set_play_volume(streamID, volume);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PLAYER, __FUNCTION__, result);
            }

            void mutePlayStreamAudio(const char *streamID, bool mute)
            {
                int result = zego_express_mute_play_stream_audio(streamID, mute);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PLAYER, __FUNCTION__, result);
            }

            void mutePlayStreamVideo(const char *streamID, bool mute)
            {
                int result = zego_express_mute_play_stream_video(streamID, mute);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PLAYER, __FUNCTION__, result);
            }

            void enableHardwareDecoder(bool enable)
            {
                int result = zego_express_enable_hardware_decoder(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PLAYER, __FUNCTION__, result);
            }

            void enableCheckPoc(bool enable)
            {
                int result = zego_express_enable_check_poc(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PUBLISHER, __FUNCTION__, result);
            }

            void muteMicrophone(bool mute)
            {
                int result = zego_express_mute_microphone(mute);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            bool isMicrophoneMuted()
            {
                bool result = zego_express_get_microphone_mute();
                return result;
            }

            void muteSpeaker(bool mute)
            {
                int result = zego_express_mute_speaker(mute);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            bool isSpeakerMuted()
            {
                bool result = zego_express_get_speaker_mute();
                return result;
            }

            void muteAudioOutput(bool mute)
            {
                int result = zego_express_mute_audio_output(mute);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void enableCamera(bool enable, enum zego_publish_channel channel)
            {
                int result = zego_express_enable_camera(enable, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void enableAudioCaptureDevice(bool enable)
            {
                int result = zego_express_enable_audio_capture_device(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void startSoundLevelMonitor()
            {
                int result = zego_express_start_sound_level_monitor();
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void stopSoundLevelMonitor()
            {
                int result = zego_express_stop_sound_level_monitor();
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void startAudioSpectrumMonitor()
            {
                int result = zego_express_start_audio_spectrum_monitor();
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void stopAudioSpectrumMonitor()
            {
                int result = zego_express_stop_audio_spectrum_monitor();
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void enableHeadphoneMonitor(bool enable)
            {
                int result = zego_express_enable_headphone_monitor(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }
            
            void setHeadphoneMonitorVolume(int volume)
            {
                int result = zego_express_set_headphone_monitor_volume(volume);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void enableMixSystemPlayout(bool enable)
            {
                int result = zego_express_enable_mix_system_playout(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            void useAudioDevice(zego_audio_device_type deviceType, const char *deviceID)
            {
                int result = zego_express_use_audio_device(deviceType, deviceID);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            zego_device_info *getAudioDeviceList(zego_audio_device_type device_type, int *device_count)
            {
                zego_device_info *result = zego_express_get_audio_device_list(device_type, device_count);
                return result;
            }

            void freeAudioDeviceList(zego_device_info *device_list)
            {
                int result = zego_express_free_audio_device_list(device_list);
                (void)result;
            }

            void useVideoDevice(const char *deviceID, enum zego_publish_channel channel)
            {
                int result = zego_express_use_video_device(deviceID, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_DEVICE, __FUNCTION__, result);
            }

            zego_device_info *getVideoDeviceList(int *device_count)
            {
                zego_device_info *result = zego_express_get_video_device_list(device_count);
                return result;
            }

            void freeVideoDeviceList(zego_device_info *device_list)
            {
                int result = zego_express_free_video_device_list(device_list);
                (void)result;
            }

            void enableAEC(bool enable)
            {
                int result = zego_express_enable_aec(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void setAECMode(zego_aec_mode mode)
            {
                int result = zego_express_set_aec_mode(mode);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void enableAGC(bool enable)
            {
                int result = zego_express_enable_agc(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void setANSMode(zego_ans_mode mode)
            {
                int result = zego_express_set_ans_mode(mode);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void enableANS(bool enable)
            {
                int result = zego_express_enable_ans(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void enableAudioMixing(bool enable)
            {
                int result = zego_express_enable_audio_mixing(enable);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void muteLocalAudioMixing(bool mute)
            {
                int result = zego_express_mute_local_audio_mixing(mute);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void setAudioMixingVolume(int volume, zego_volume_type type)
            {
                int result = zego_express_set_audio_mixing_volume_with_type(volume, type);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void enableBeautify(int feature, enum zego_publish_channel channel)
            {
                int result = zego_express_enable_beautify(feature, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void setBeautifyOption(zego_beautify_option option, enum zego_publish_channel channel)
            {
                int result = zego_express_set_beautify_option(option, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void setVoiceChangerParam(float param)
            {
                int result = zego_express_set_voice_changer_param(param);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void enableVirtualStereo(bool enable, int angle)
            {
                int result = zego_express_enable_virtual_stereo(enable, angle);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            void setReverbParam(struct zego_reverb_param param)
            {
                int result = zego_express_set_reverb_param(param);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_PREPROCESS, __FUNCTION__, result);
            }

            int sendBroadcastMessage(const char *room_id, const char *content)
            {
                int seq = zego_express_send_broadcast_message(room_id, content);
                return seq;
            }

            int sendBarrageMessage(const char *room_id, const char *content)
            {
                int seq = zego_express_send_barrage_message(room_id, content);
                return seq;
            }

            int sendCustomCommand(const char *room_id, const char *content, struct zego_user *to_user_list, unsigned int to_user_count)
            {
                int seq = zego_express_send_custom_command(room_id, content, to_user_list, to_user_count);
                return seq;
            }

            int startMixerTask(zego_mixer_task task)
            {
                int seq = zego_express_start_mixer_task(task);
                return seq;
            }

            int stopMixerTask(zego_mixer_task task)
            {
                int seq = zego_express_stop_mixer_task(task);
                return seq;
            }

            int startAutoMixerTask(zego_auto_mixer_task task)
            {
                int seq = zego_express_start_auto_mixer_task(task);
                return seq;
            }

            int stopAutoMixerTask(zego_auto_mixer_task task)
            {
                int seq = zego_express_stop_auto_mixer_task(task);
                return seq;
            }

            zego_mediaplayer_instance_index createMediaPlayer()
            {
                zego_mediaplayer_instance_index instanceIndex = zego_express_create_mediaplayer();
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, instanceIndex == zego_mediaplayer_instance_index_null ? ZEGO_ERRCODE_MEDIAPLAYER_EXCEED_MAX_COUNT : ZEGO_ERRCODE_COMMON_SUCCESS);
                return instanceIndex;
            }

            int destroyMediaPlayer(zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_release_mediaplayer(instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerStart(zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_start(instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerStop(zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_stop(instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerPause(zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_pause(instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerResume(zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_resume(instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerPreload(const char *path, zego_mediaplayer_instance_index instance_index)
            {
                int seq = zego_express_mediaplayer_load_resource(path, instance_index);
                return seq;
            }

            int mediaPlayerSeekTo(unsigned long long millisecond, zego_mediaplayer_instance_index instance_index)
            {
                int seq = zego_express_mediaplayer_seek_to(millisecond, instance_index);
                return seq;
            }

            int mediaPlayerSetVolume(int volume, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_set_volume(volume, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            unsigned long long mediaPlayerGetTotalDuration(zego_mediaplayer_instance_index instance_index)
            {
                unsigned long long totalDuration = zego_express_mediaplayer_get_total_duration(instance_index);
                return totalDuration;
            }

            unsigned long long mediaPlayerGetCurrentProgress(zego_mediaplayer_instance_index instance_index)
            {
                unsigned long long currentProgress = zego_express_mediaplayer_get_current_duration(instance_index);
                return currentProgress;
            }

            int mediaPlayerGetVolume(zego_mediaplayer_instance_index instance_index)
            {
                int volume = zego_express_mediaplayer_get_volume(instance_index);
                return volume;
            }

            zego_mediaplayer_state mediaPlayerGetCurrentState(zego_mediaplayer_instance_index instance_index)
            {
                zego_mediaplayer_state state = zego_express_mediaplayer_get_state(instance_index);
                return state;
            }

            int mediaPlayerMuteLocal(bool mute, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_mute_local_audio(mute, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerSetPlayerCanvas(struct zego_canvas *canvas, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_set_player_canvas(canvas, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerEnableAUX(bool enable, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_enable_aux(enable, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerEnableRepeat(bool enable, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_enable_repeat(enable, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerSetProgressInterval(unsigned long long millisecond, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_set_progress_interval(millisecond, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerEnableAudioData(bool enable, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_enable_audio_data(enable, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }

            int mediaPlayerEnableVideoData(bool enable, zego_video_frame_format format, zego_mediaplayer_instance_index instance_index)
            {
                int result = zego_express_mediaplayer_enable_video_data(enable, format, instance_index);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_MEDIAPLAYER, __FUNCTION__, result);
                return result;
            }


            void enableCustomVideoRender(bool enable, struct zego_custom_video_render_config* config)
            {
                int result = zego_express_enable_custom_video_render(enable, config);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_CUSTOMVIDEOIO, __FUNCTION__, result);
            }

            void enableCustomVideoCapture(bool enable, zego_custom_video_capture_config* config, zego_publish_channel channel)
            {
                int result = zego_express_enable_custom_video_capture(enable, config, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_CUSTOMVIDEOIO, __FUNCTION__, result);
            }

            void customVideoCaptureSetFillMode(zego_view_mode mode, enum zego_publish_channel channel)
            {
                int result = zego_express_custom_video_capture_set_fill_mode(mode, channel);
                printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_CUSTOMVIDEOIO, __FUNCTION__, result);
            }

            void customVideoCaptureSendRawData(const unsigned char *data, unsigned int data_length, const struct zego_video_frame_param param, unsigned long long reference_time_millisecond, enum zego_publish_channel channel)
            {
                int result = zego_express_custom_video_capture_send_memory_data(data, data_length, param, reference_time_millisecond, 1000, channel);
                if (result != 0)
                {
                    printDebugInfo(ZEGO_EXPRESS_MODULE_CUSTOMVIDEOIO, __FUNCTION__, result);
                }
            }

            void customVideoCaptureSendEncodedFrameData(const unsigned char *data, unsigned int data_length, const struct zego_video_encoded_frame_param param, unsigned long long reference_time_millisecond, enum zego_publish_channel channel)
            {
                int result = zego_express_custom_video_capture_send_encoded_frame_data(data, data_length, param, double(reference_time_millisecond), channel);
                if (result != 0)
                {
                    printDebugInfo(ZEGO_EXPRESS_MODULE_CUSTOMVIDEOIO, __FUNCTION__, result);
                }
            }

			void enableAudioDataCallback(bool enable, unsigned int callback_bitmask, zego_audio_frame_param param)
			{
				int result = zego_express_enable_audio_data_callback(enable, callback_bitmask, param);
				printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_CUSTOMAUDIOIO, __FUNCTION__, result);
			}

			void startRecordingCapturedData(zego_data_record_config config, zego_publish_channel channel)
			{
				int result = zego_express_start_recording_captured_data(config, channel);
				printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_RECORD, __FUNCTION__, result);
			}

			void stopRecordingCapturedData(zego_publish_channel channel)
			{
				int result = zego_express_stop_recording_captured_data(channel);
				printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_RECORD, __FUNCTION__, result);
			}

			void enableCustomAudioIO(bool enable, zego_custom_audio_config *config, zego_publish_channel channel)
			{
				int result = zego_express_enable_custom_audio_io(enable, config, channel);
				printDebugInfoAndPopupWhenError(ZEGO_EXPRESS_MODULE_RECORD, __FUNCTION__, result);
			}

			void sendCustomAudioCaptureAACData(unsigned char * data, unsigned int dataLength, unsigned int configLength, unsigned long long referenceTimeMillisecond, zego_audio_frame_param param, zego_publish_channel channel)
			{
				int result = zego_express_send_custom_audio_capture_aac_data(data, dataLength, configLength, referenceTimeMillisecond, param, channel);
				if (result != 0)
				{
					printDebugInfo(ZEGO_EXPRESS_MODULE_CUSTOMAUDIOIO, __FUNCTION__, result);
				}
			}

			void sendCustomAudioCapturePCMData(unsigned char * data, unsigned int dataLength, zego_audio_frame_param param, zego_publish_channel channel)
			{
				int result = zego_express_send_custom_audio_capture_pcm_data(data, dataLength, param, channel);
				if (result != 0)
				{
					printDebugInfo(ZEGO_EXPRESS_MODULE_CUSTOMAUDIOIO, __FUNCTION__, result);
				}
			}

			void fetchCustomAudioRenderPCMData(unsigned char * data, unsigned int dataLength, zego_audio_frame_param param)
			{
				int result = zego_express_fetch_custom_audio_render_pcm_data(data, dataLength, param);
				if (result != 0)
				{
					printDebugInfo(ZEGO_EXPRESS_MODULE_CUSTOMAUDIOIO, __FUNCTION__, result);
				}
			}

            void printDebugInfoAndPopupWhenError(int module, const char *function_name, int error_code)
            {
                auto detail = zego_express_get_print_debug_info(module, function_name, error_code);
                if (error_code != 0 && pri->isTestEnv && pri->enableToastOnTestEnv)
                {
                    zego_express_show_debug_message_box(pri->debugVerboseLanguage, error_code, function_name, detail);
                }
                if (pri->enableDebugErrorAlways || error_code != 0)
                {
                    zego_express_trigger_on_debug_error(error_code, function_name, detail);
                }
            }

            void printDebugInfo(int module, const char *function_name, int error_code)
            {
                auto detail = zego_express_get_print_debug_info(module, function_name, error_code);
                if (pri->enableDebugErrorAlways || error_code != 0)
                {
                    zego_express_trigger_on_debug_error(error_code, function_name, detail);
                }
            }

            void postWindowsMessage(void* message)
            {
                zego_express_post_windows_message(message);
            }

            // eventHandler
            void registerRecvWindowsMessageCallback(void* callback, void* user_context)
            {
                zego_register_recv_windows_message_callback(callback, user_context);
            }

            void registerEngineStateCallback(void *callback_func, void *user_context)
            {
                zego_register_engine_state_update_callback(zego_on_engine_state_update(callback_func), user_context);
            }

            void registerEngineUninitCallback(void* callback_func, void* user_context)
            {
                zego_register_engine_uninit_callback(zego_on_engine_uninit(callback_func), user_context);
            }

            void registerOnCallErrorCallback(void *callback, void *context)
            {
                zego_register_debug_error_callback(zego_on_debug_error(callback), context);
            }

            void registerRoomStreamUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_room_stream_update_callback(zego_on_room_stream_update(callback_func), user_context);
            }

            void registerRoomStreamExtraInfoUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_room_stream_extra_info_update_callback(zego_on_room_stream_extra_info_update(callback_func), user_context);
            }

            void registerRoomStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_room_state_update_callback(zego_on_room_state_update(callback_func), user_context);
            }

            void registerRoomUserStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_room_user_state_update_callback(zego_on_room_user_update(callback_func), user_context);
            }

            void registerRoomOnlineUserCountUpdateCallback(void *callback_func, void* user_context){
                zego_register_room_online_user_count_update_callback(zego_on_room_online_user_count_update(callback_func), user_context);
            }

            void registerPublisherStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_state_update_callback(zego_on_publisher_state_update(callback_func), user_context);
            }

            void registerPublisherQualityUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_quality_update_callback(zego_on_publisher_quality_update(callback_func), user_context);
            }

            void registerPublisherRecvAudioCapturedFirstFrameCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_recv_audio_captured_first_frame_callback(zego_on_publisher_recv_audio_captured_first_frame(callback_func), user_context);
            }

            void registerPublisherRecvVideoCapturedFirstFrameCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_recv_video_captured_first_frame_callback(zego_on_publisher_recv_video_captured_first_frame(callback_func), user_context);
            }

            void registerPublisherVideoSizeChangedCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_video_size_changed_callback(zego_on_publisher_video_size_changed(callback_func), user_context);
            }

            void registerPublisherRelayCDNStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_relay_cdn_state_update_callback(zego_on_publisher_relay_cdn_state_update(callback_func), user_context);
            }

            void registerPublisherUpdatePublishCdnUrlCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_update_publish_cdn_url_callback(zego_on_publisher_update_cdn_url_result(callback_func), user_context);
            }

            void registerPublisherUpdateStreamExtraInfoResultCallback(void *callback_func, void *user_context)
            {
                zego_register_publisher_update_stream_extra_info_result_callback(zego_on_publisher_update_stream_extra_info_result(callback_func), user_context);
            }

            void registerPlayerStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_player_state_update_callback(zego_on_player_state_update(callback_func), user_context);
            }

            void registerPlayerQualityUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_player_quality_update_callback(zego_on_player_quality_update(callback_func), user_context);
            }

            void registerPlayerMediaEventCallback(void *callback_func, void *user_context)
            {
                zego_register_player_media_event_callback(zego_on_player_media_event(callback_func), user_context);
            }

            void registerPlayerRecvAudioFirstFrameCallback(void *callback_func, void *user_context)
            {
                zego_register_player_recv_audio_first_frame_callback(zego_on_player_recv_audio_first_frame(callback_func), user_context);
            }

            void registerPlayerRecvVideoFirstFrameCallback(void *callback_func, void *user_context)
            {
                zego_register_player_recv_video_first_frame_callback(zego_on_player_recv_video_first_frame(callback_func), user_context);
            }

            void registerPlayerRenderVideoFirstFrameCallback(void *callback_func, void *user_context)
            {
                zego_register_player_render_video_first_frame_callback(zego_on_player_render_video_first_frame(callback_func), user_context);
            }

            void registerPlayerVideoSizeChangedCallback(void *callback_func, void *user_context)
            {
                zego_register_player_video_size_changed_callback(zego_on_player_video_size_changed(callback_func), user_context);
            }

            void registerPlayerRecvSEICallback(void *callback_func, void *user_context)
            {
                zego_register_player_recv_sei_callback(zego_on_player_recv_sei(callback_func), user_context);
            }

            void registerCapturedSoundLevelUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_captured_sound_level_update_callback(zego_on_captured_sound_level_update(callback_func), user_context);
            }

            void registerRemoteSoundLevelUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_remote_sound_level_update_callback(zego_on_remote_sound_level_update(callback_func), user_context);
            }

            void registerCapturedAudioSpectrumUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_captured_audio_spectrum_update_callback(zego_on_captured_audio_spectrum_update(callback_func), user_context);
            }

            void registerRemoteAudioSpectrumUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_remote_audio_spectrum_update_callback(zego_on_remote_audio_spectrum_update(callback_func), user_context);
            }

            void registerAudioDeviceStateChangedCallback(void *callback_func, void *user_context)
            {
                zego_register_audio_device_state_changed_callback(zego_on_audio_device_state_changed(callback_func), user_context);
            }

            void registerVideoDeviceStateChangedCallback(void *callback_func, void *user_context)
            {
                zego_register_video_device_state_changed_callback(zego_on_video_device_state_changed(callback_func), user_context);
            }

            void registerDeviceErrorCallback(void *callback_func, void *user_context)
            {
                zego_register_device_error_callback(zego_on_device_error(callback_func), user_context);
            }

            void registerRemoteCameraStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_remote_camera_state_update_callback(zego_on_remote_camera_state_update(callback_func), user_context);
            }

            void registerRemoteMicStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_remote_mic_state_update_callback(zego_on_remote_mic_state_update(callback_func), user_context);
            }

            void registerIMSendBroadcastMessageResultCallback(void *callback_func, void *user_context)
            {
                zego_register_im_send_broadcast_message_result_callback(zego_on_im_send_broadcast_message_result(callback_func), user_context);
            }

            void registerIMRecvBroadcasMessageCallback(void *callback_func, void *user_context)
            {
                zego_register_im_recv_broadcast_message_callback(zego_on_im_recv_broadcast_message(callback_func), user_context);
            }

            void registerIMSendBarrageMessageResultCallback(void *callback_func, void *user_context)
            {
                zego_register_im_send_barrage_message_result_callback(zego_on_im_send_barrage_message_result(callback_func), user_context);
            }

            void registerIMRecvBarrageMessageResultCallback(void *callback_func, void *user_context)
            {
                zego_register_im_recv_barrage_message_callback(zego_on_im_recv_barrage_message(callback_func), user_context);
            }

            void registerIMSendCustomCommandResultCallback(void *callback_func, void *user_context)
            {
                zego_register_im_send_custom_command_result_callback(zego_on_im_send_custom_command_result(callback_func), user_context);
            }

            void registerIMRecvCustomCommandCallback(void *callback_func, void *user_context)
            {
                zego_register_im_recv_custom_command_callback(zego_on_im_recv_custom_command(callback_func), user_context);
            }

            void registerMixerStartResultCallback(void *callback_func, void *user_context)
            {
                zego_register_mixer_start_result_callback(zego_on_mixer_start_result(callback_func), user_context);
                zego_register_auto_mixer_start_result_callback(zego_on_auto_mixer_start_result(callback_func), user_context);
            }

            void registerMixerStopResultCallback(void *callback_func, void *user_context)
            {
                zego_register_mixer_stop_result_callback(zego_on_mixer_stop_result(callback_func), user_context);
                zego_register_auto_mixer_stop_result_callback(zego_on_auto_mixer_stop_result(callback_func), user_context);
            }

            void registerMixerSoundLevelUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_mixer_sound_level_update(zego_on_mixer_sound_level_update(callback_func), user_context);
            }

            void registerAutoMixerSoundLevelUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_auto_mixer_sound_level_update(zego_on_auto_mixer_sound_level_update(callback_func), user_context);
            }

            void registerMixerRelayCDNStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_mixer_relay_cdn_state_update_callback(zego_on_mixer_relay_cdn_state_update(callback_func), user_context);
            }

            void registerMediaPlayerStateUpdateCallback(void *callback_func, void *user_context)
            {
                zego_register_mediaplayer_state_update_callback(zego_on_mediaplayer_state_update(callback_func), user_context);
            }

            void registerMediaPlayerNetworkEventCallback(void *callback_func, void *user_context)
            {
                zego_register_mediaplayer_network_event_callback(zego_on_mediaplayer_network_event(callback_func), user_context);
            }

            void registerMediaPlayerPlayingProgressCallback(void *callback_func, void *user_context)
            {
                zego_register_mediaplayer_playing_progress_callback(zego_on_mediaplayer_playing_progress(callback_func), user_context);
            }

            void registerMediaPlayerSeekToResult(void *callback_func, void *user_context)
            {
                zego_register_mediaplayer_seek_to_result(zego_on_mediaplayer_seek_to_time_result(callback_func), user_context);
            }

            void registerMediaPlayerLoadFileResult(void *callback_func, void *user_context)
            {
                zego_register_mediaplayer_load_resource_result(zego_on_mediaplayer_load_resource_result(callback_func), user_context);
            }

            void registerMediaPlayerAudioDataCallback(void *callback_func, void *user_context)
            {
                zego_register_mediaplayer_audio_data_callback(zego_on_mediaplayer_audio_data(callback_func), user_context);
            }

            void registerMediaPlayerVideoDataCallback(void *callback_func, void *user_context)
            {
                zego_register_mediaplayer_video_data_callback(zego_on_mediaplayer_video_data(callback_func), user_context);
            }

            void registerCustomVideoRenderLocalFrameDataCallback(void *callback_func, void *user_context)
            {
                zego_register_custom_video_render_captured_frame_data_callback(zego_on_custom_video_render_captured_frame_data(callback_func), user_context);
            }

            void registerCustomVideoRenderRemoteFrameDataCallback(void *callback_func, void *user_context)
            {
                zego_register_custom_video_render_remote_frame_data_callback(zego_on_custom_video_render_remote_frame_data(callback_func), user_context);
            }

            void registerCustomVideoRenderLocalFrameEncodedDataCallback(void *callback_func, void *user_context)
            {
                zego_register_custom_video_render_captured_frame_encoded_data_callback(zego_on_custom_video_render_captured_frame_encoded_data(callback_func), user_context);
            }

            void registerCustomVideoRenderRemoteFrameEncodedDataCallback(void *callback_func, void *user_context)
            {
                zego_register_custom_video_render_remote_frame_encoded_data_callback(zego_on_custom_video_render_remote_frame_encoded_data(callback_func), user_context);
            }

            void registerCustomVideoCaptureStartCallback(void *callback_func, void *user_context)
            {
                zego_register_custom_video_capture_start_callback(zego_on_custom_video_capture_start(callback_func), user_context);
            }

            void registerCustomVideoCaptureStopCallback(void *callback_func, void *user_context)
            {
                zego_register_custom_video_capture_stop_callback(zego_on_custom_video_capture_stop(callback_func), user_context);
            }

            void registerAudioMixingCopyDataCallback(void *callback_func, void* user_context)
			{
                zego_register_audio_mixing_copy_data_callback(zego_on_copy_audio_mixing_data(callback_func), user_context);
            }

			void registerOnCapturedAudioDataCallback(void *callback_func, void* user_context)
			{
				zego_register_on_captured_audio_data_callback(zego_on_captured_audio_data(callback_func), user_context);
			}

			void registerOnRemoteAudioDataCallback(void *callback_func, void* user_context)
			{
				zego_register_on_remote_audio_data_callback(zego_on_remote_audio_data(callback_func), user_context);
			}

			void registerRecordingCapturedDataStateUpdateCallback(void *callback_func, void* user_context)
			{
				zego_register_recording_captured_data_state_update_callback(zego_on_captured_media_recorder_state_update(callback_func), user_context);
			}

			void registerRecordingCapturedDataProgressUpdateCallback(void *callback_func, void* user_context)
			{
				zego_register_recording_captured_data_progress_update_callback(zego_on_captured_media_record_progress_update(callback_func), user_context);
			}

			void registerOnMixedAudioDataCallback(void *callback_func, void* user_context)
			{
				zego_register_on_mixed_audio_data_callback(zego_on_mixed_audio_data(callback_func), user_context);
			}

        private:
            ZegoExpressEngineBridge()
            {
                pri = new ZegoExpressEngineBridgePri();
            }

            ~ZegoExpressEngineBridge()
            {
                delete pri;
            }

            ZegoExpressEngineBridgePri* pri = nullptr;
        };
        #define oInternalOriginBridge ZegoExpressEngineBridge::GetInstance()

    }
}

#endif /* __ZEGOEXPRESSBRIDGE_H__ */
