﻿
#pragma once

#include "../ZegoExpressInterface.h"
#include "../ZegoExpressDefines.h"
#include "../ZegoExpressEventHandler.h"

#include "ZegoInternalBase.h"
#include "ZegoInternalBridge.h"
#include "ZegoInternalMediaPlayer.hpp"

namespace ZEGO
{
    namespace EXPRESS
    {

        #define oInternalCallbackCenter ZegoSingleton<ZegoInternalCallbackCenter>::CreateInstance()
        class ZegoInternalCallbackCenter
        {
        public:
			declearSingleShareMember(IZegoEventHandler)
				declearMultiRawMember(ZegoPublisherSetStreamExtraInfoCallback)
				declearMultiRawMember(ZegoPublisherUpdateCdnUrlCallback)
				declearMultiRawMember(ZegoIMSendBroadcastMessageCallback)
				declearMultiRawMember(ZegoIMSendBarrageMessageCallback)
				declearMultiRawMember(ZegoIMSendCustomCommandCallback)
				declearMultiRawMember(ZegoMixerStartCallback)
				declearMultiRawMember(ZegoMixerStopCallback)
				declearMultiShareMember(ZegoExpressMediaPlayerImp)
#if ZEGO_EXPRESS_VIDEO_SDK
				declearSingleShareMember(IZegoCustomVideoRenderHandler)
				declearSingleShareMember(IZegoCustomVideoCaptureHandler)
#endif
				declearSingleShareMember(IZegoAudioDataHandler)
				declearSingleShareMember(IZegoAudioMixingHandler)
				declearSingleShareMember(IZegoDataRecordEventHandler);
                declearSingleRawMember(ZegoDestroyCompletionCallback)

            void clearHandlerData()
            {
                mIZegoEventHandler = nullptr;
                mZegoPublisherSetStreamExtraInfoCallback.clear();
                mZegoPublisherUpdateCdnUrlCallback.clear();
                mZegoIMSendBroadcastMessageCallback.clear();
                mZegoIMSendBarrageMessageCallback.clear();
                mZegoIMSendCustomCommandCallback.clear();
                mZegoMixerStartCallback.clear();
                mZegoMixerStopCallback.clear();
                mZegoExpressMediaPlayerImp.clear();
#if ZEGO_EXPRESS_VIDEO_SDK
                mIZegoCustomVideoRenderHandler = nullptr;
                mIZegoCustomVideoCaptureHandler = nullptr;
#endif
                mIZegoAudioDataHandler = nullptr;
                mIZegoAudioMixingHandler = nullptr;
				mIZegoDataRecordEventHandler = nullptr;
            }

            void registerCallback()
            {
#ifdef WIN32
                oInternalOriginBridge->registerRecvWindowsMessageCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_recv_win_massage), nullptr);
#endif
                oInternalOriginBridge->registerEngineUninitCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_engine_uninit), nullptr);

                oInternalOriginBridge->registerOnCallErrorCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_debug_error), ZegoVoidPtr(this));
                oInternalOriginBridge->registerEngineStateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_engine_state_update), ZegoVoidPtr(this));

                oInternalOriginBridge->registerRoomStreamUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_stream_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerRoomStreamExtraInfoUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_room_stream_extra_info_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerRoomStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_room_state_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerRoomUserStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_user_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerRoomOnlineUserCountUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_room_online_user_count_update), ZegoVoidPtr(this));

                oInternalOriginBridge->registerPublisherStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_state_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPublisherQualityUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_quality_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPublisherRecvAudioCapturedFirstFrameCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_recv_audio_captured_first_frame), ZegoVoidPtr(this));
#if ZEGO_EXPRESS_VIDEO_SDK
                oInternalOriginBridge->registerPublisherRecvVideoCapturedFirstFrameCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_recv_video_captured_first_frame), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPublisherVideoSizeChangedCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_video_size_changed), ZegoVoidPtr(this));
#endif
                oInternalOriginBridge->registerPublisherUpdatePublishCdnUrlCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_update_cdn_url_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPublisherRelayCDNStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_relay_cdn_state_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPublisherUpdateStreamExtraInfoResultCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_publisher_update_stream_extra_info_result), ZegoVoidPtr(this));

                oInternalOriginBridge->registerPlayerStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_state_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPlayerQualityUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_quality_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPlayerMediaEventCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_media_event), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPlayerRecvAudioFirstFrameCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_recv_audio_first_frame), ZegoVoidPtr(this));
#if ZEGO_EXPRESS_VIDEO_SDK
                oInternalOriginBridge->registerPlayerRecvVideoFirstFrameCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_recv_video_first_frame), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPlayerRenderVideoFirstFrameCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_render_video_first_frame), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPlayerVideoSizeChangedCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_video_size_changed), ZegoVoidPtr(this));
                oInternalOriginBridge->registerPlayerRecvSEICallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_player_recv_sei), ZegoVoidPtr(this));
#endif

                oInternalOriginBridge->registerDeviceErrorCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_device_error), ZegoVoidPtr(this));
                oInternalOriginBridge->registerAudioDeviceStateChangedCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_audio_device_state_changed), ZegoVoidPtr(this));
#if ZEGO_EXPRESS_VIDEO_SDK
                oInternalOriginBridge->registerVideoDeviceStateChangedCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_video_device_state_changed), ZegoVoidPtr(this));
                oInternalOriginBridge->registerRemoteCameraStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_remote_camera_state_update), ZegoVoidPtr(this));
#endif
                oInternalOriginBridge->registerRemoteMicStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_remote_mic_state_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerCapturedSoundLevelUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_captured_sound_level_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerRemoteSoundLevelUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_remote_sound_level_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerCapturedAudioSpectrumUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_captured_audio_spectrum_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerRemoteAudioSpectrumUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_remote_audio_spectrum_update), ZegoVoidPtr(this));

                oInternalOriginBridge->registerIMSendBroadcastMessageResultCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_im_send_broadcast_message_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerIMRecvBroadcasMessageCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_im_recv_broadcast_message), ZegoVoidPtr(this));
                oInternalOriginBridge->registerIMSendBarrageMessageResultCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_im_send_barrage_message_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerIMRecvBarrageMessageResultCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_im_recv_barrage_message), ZegoVoidPtr(this));
                oInternalOriginBridge->registerIMSendCustomCommandResultCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_im_send_custom_command_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerIMRecvCustomCommandCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_im_recv_custom_command), ZegoVoidPtr(this));

                oInternalOriginBridge->registerMixerStartResultCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mixer_start_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMixerStopResultCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mixer_stop_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMixerSoundLevelUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mixer_sound_level_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMixerRelayCDNStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mixer_relay_cdn_state_update), ZegoVoidPtr(this));

                oInternalOriginBridge->registerMediaPlayerStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mediaplayer_state_update), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMediaPlayerNetworkEventCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mediaplayer_network_event), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMediaPlayerPlayingProgressCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mediaplayer_playing_progress), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMediaPlayerSeekToResult(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mediaplayer_seek_to_time_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMediaPlayerLoadFileResult(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mediaplayer_load_file_result), ZegoVoidPtr(this));
                oInternalOriginBridge->registerMediaPlayerAudioDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mediaplayer_audio_data), ZegoVoidPtr(this));

#if ZEGO_EXPRESS_VIDEO_SDK
                oInternalOriginBridge->registerMediaPlayerVideoDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mediaplayer_video_data), ZegoVoidPtr(this));
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
                oInternalOriginBridge->registerCustomVideoRenderLocalFrameDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_custom_video_render_captured_frame_data), ZegoVoidPtr(this));
                oInternalOriginBridge->registerCustomVideoRenderRemoteFrameDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_custom_video_render_remote_frame_data), ZegoVoidPtr(this));
                oInternalOriginBridge->registerCustomVideoRenderLocalFrameEncodedDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_custom_video_render_captured_frame_encoded_data), ZegoVoidPtr(this));
                oInternalOriginBridge->registerCustomVideoRenderRemoteFrameEncodedDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_custom_video_render_remote_frame_encoded_data), ZegoVoidPtr(this));
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
                oInternalOriginBridge->registerCustomVideoCaptureStartCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_custom_video_capture_start), ZegoVoidPtr(this));
                oInternalOriginBridge->registerCustomVideoCaptureStopCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_custom_video_capture_stop), ZegoVoidPtr(this));
#endif

                oInternalOriginBridge->registerAudioMixingCopyDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_audio_mixing_copy_data), ZegoVoidPtr(this));

				oInternalOriginBridge->registerOnCapturedAudioDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_captured_audio_data), ZegoVoidPtr(this));
				oInternalOriginBridge->registerOnRemoteAudioDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_remote_audio_data), ZegoVoidPtr(this));
				oInternalOriginBridge->registerOnMixedAudioDataCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_mixed_audio_data), ZegoVoidPtr(this));

				oInternalOriginBridge->registerRecordingCapturedDataStateUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_captured_data_record_state_update), ZegoVoidPtr(this));
				oInternalOriginBridge->registerRecordingCapturedDataProgressUpdateCallback(ZegoVoidPtr(&ZegoInternalCallbackCenter::zego_on_captured_data_record_progress_update), ZegoVoidPtr(this));

            }

            void unregisterCallback()
            {
                oInternalOriginBridge->registerOnCallErrorCallback(nullptr, nullptr);
                oInternalOriginBridge->registerEngineStateCallback(nullptr, nullptr);

                oInternalOriginBridge->registerRoomStreamUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRoomStreamExtraInfoUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRoomStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRoomUserStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRoomOnlineUserCountUpdateCallback(nullptr, nullptr);

                oInternalOriginBridge->registerPublisherStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPublisherQualityUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPublisherRecvAudioCapturedFirstFrameCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPublisherRecvVideoCapturedFirstFrameCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPublisherVideoSizeChangedCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPublisherUpdatePublishCdnUrlCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPublisherRelayCDNStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPublisherUpdateStreamExtraInfoResultCallback(nullptr, nullptr);

                oInternalOriginBridge->registerPlayerStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPlayerQualityUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPlayerMediaEventCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPlayerRecvAudioFirstFrameCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPlayerRecvVideoFirstFrameCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPlayerRenderVideoFirstFrameCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPlayerVideoSizeChangedCallback(nullptr, nullptr);
                oInternalOriginBridge->registerPlayerRecvSEICallback(nullptr, nullptr);

                oInternalOriginBridge->registerAudioDeviceStateChangedCallback(nullptr, nullptr);
                oInternalOriginBridge->registerVideoDeviceStateChangedCallback(nullptr, nullptr);
                oInternalOriginBridge->registerDeviceErrorCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRemoteCameraStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRemoteMicStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerCapturedSoundLevelUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRemoteSoundLevelUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerCapturedAudioSpectrumUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerRemoteAudioSpectrumUpdateCallback(nullptr, nullptr);

                oInternalOriginBridge->registerIMSendBroadcastMessageResultCallback(nullptr, nullptr);
                oInternalOriginBridge->registerIMRecvBroadcasMessageCallback(nullptr, nullptr);
                oInternalOriginBridge->registerIMSendBarrageMessageResultCallback(nullptr, nullptr);
                oInternalOriginBridge->registerIMRecvBarrageMessageResultCallback(nullptr, nullptr);
                oInternalOriginBridge->registerIMSendCustomCommandResultCallback(nullptr, nullptr);
                oInternalOriginBridge->registerIMRecvCustomCommandCallback(nullptr, nullptr);

                oInternalOriginBridge->registerMixerStartResultCallback(nullptr, nullptr);
                oInternalOriginBridge->registerMixerStopResultCallback(nullptr, nullptr);
                oInternalOriginBridge->registerMixerSoundLevelUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerAutoMixerSoundLevelUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerMixerRelayCDNStateUpdateCallback(nullptr, nullptr);

                oInternalOriginBridge->registerMediaPlayerStateUpdateCallback(nullptr, nullptr);
                oInternalOriginBridge->registerMediaPlayerNetworkEventCallback(nullptr, nullptr);
                oInternalOriginBridge->registerMediaPlayerPlayingProgressCallback(nullptr, nullptr);
                oInternalOriginBridge->registerMediaPlayerSeekToResult(nullptr, nullptr);
                oInternalOriginBridge->registerMediaPlayerLoadFileResult(nullptr, nullptr);
                oInternalOriginBridge->registerMediaPlayerAudioDataCallback(nullptr, nullptr);
                oInternalOriginBridge->registerMediaPlayerVideoDataCallback(nullptr, nullptr);

                oInternalOriginBridge->registerCustomVideoRenderLocalFrameDataCallback(nullptr, nullptr);
                oInternalOriginBridge->registerCustomVideoRenderRemoteFrameDataCallback(nullptr, nullptr);
                oInternalOriginBridge->registerCustomVideoRenderLocalFrameEncodedDataCallback(nullptr, nullptr);
                oInternalOriginBridge->registerCustomVideoRenderRemoteFrameEncodedDataCallback(nullptr, nullptr);

                oInternalOriginBridge->registerCustomVideoCaptureStartCallback(nullptr, nullptr);
                oInternalOriginBridge->registerCustomVideoCaptureStopCallback(nullptr, nullptr);

                oInternalOriginBridge->registerAudioMixingCopyDataCallback(nullptr, nullptr);

				oInternalOriginBridge->registerOnCapturedAudioDataCallback(nullptr, nullptr);
				oInternalOriginBridge->registerOnRemoteAudioDataCallback(nullptr, nullptr);
				oInternalOriginBridge->registerOnMixedAudioDataCallback(nullptr, nullptr);

				oInternalOriginBridge->registerRecordingCapturedDataStateUpdateCallback(nullptr, nullptr);
				oInternalOriginBridge->registerRecordingCapturedDataProgressUpdateCallback(nullptr, nullptr);
            }

            static void zego_on_engine_uninit(void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto afterDestroy = oInternalCallbackCenter->getZegoDestroyCompletionCallback();
                if (afterDestroy)
                {
                    afterDestroy();
                    oInternalCallbackCenter->setZegoDestroyCompletionCallback(nullptr);
                }
            }

#ifdef WIN32
            static void zego_on_recv_win_massage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
            {
                ZEGO_UNUSED_VARIABLE(hWnd);
                ZEGO_UNUSED_VARIABLE(lParam);
                if (uMsg == (WM_APP + 10086))
                {
                    std::function<void(void)> *pFunc = (std::function<void(void)> *)wParam;
                    (*pFunc)();
                    delete pFunc;
                }
            }
#endif

            // DEBUG CALLBACK
            static void zego_on_debug_error(int error_code, const char *func_name, const char *_info, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string funcName = func_name;
                std::string info = _info;
                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onDebugError(error_code, funcName, info);
                ZEGO_SWITCH_THREAD_ING
            }

            // ENEING CALLBACK
            static void zego_on_engine_state_update(zego_engine_state state, void* user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onEngineStateUpdate(ZegoEngineState(state));
                ZEGO_SWITCH_THREAD_ING
            }

            // ROOM CALLBACK
            static void zego_on_room_state_update(const char *room_id, zego_room_state state, zego_error error_code, const char *extend_data, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string roomID = room_id;
                std::string extendData = extend_data;
                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRoomStateUpdate(roomID, ZegoRoomState(state), error_code, extendData);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_stream_update(const char *room_id, zego_update_type update_type, const struct zego_stream *stream_info_list, unsigned int stream_info_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string roomID = room_id;
                std::vector<ZegoStream> streamList;
                for (unsigned int i = 0; i < stream_info_count; i++)
                {
                    ZegoStream stream = ZegoExpressConvert::I2OStream(stream_info_list[i]);
                    streamList.push_back(stream);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRoomStreamUpdate(roomID, ZegoUpdateType(update_type), streamList);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_room_stream_extra_info_update(const char *room_id, const struct zego_stream *stream_info_list, unsigned int stream_info_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string roomID = room_id;
                std::vector<ZegoStream> streamList;
                for (unsigned int i = 0; i < stream_info_count; i++)
                {
                    ZegoStream stream = ZegoExpressConvert::I2OStream(stream_info_list[i]);
                    streamList.push_back(stream);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRoomStreamExtraInfoUpdate(roomID, streamList);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_user_update(const char *room_id, zego_update_type update_type, const struct zego_user *user_list, unsigned int user_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string roomID = room_id;
                std::vector<ZegoUser> userList;
                for (unsigned int i = 0; i < user_count; i++)
                {
                    ZegoUser user = ZegoExpressConvert::I2OUser(user_list[i]);
                    userList.push_back(user);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRoomUserUpdate(roomID, ZegoUpdateType(update_type), userList);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_room_online_user_count_update(const char *room_id, int online_user_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string roomID = room_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRoomOnlineUserCountUpdate(roomID, online_user_count);
                ZEGO_SWITCH_THREAD_ING
            }

            // PUBLISHER CALLBACK
            static void zego_on_publisher_state_update(const char *stream_id, zego_publisher_state state, zego_error error_code, const char *extend_data, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;
                std::string extendData = extend_data;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPublisherStateUpdate(streamID, ZegoPublisherState(state), error_code, extendData);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_publisher_quality_update(const char *stream_id, zego_publish_stream_quality quality, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                ZegoPublishStreamQuality pushlishQuality = ZegoExpressConvert::I2OPushlishQuality(quality);
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPublisherQualityUpdate(streamID, pushlishQuality);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_publisher_recv_audio_captured_first_frame(void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPublisherCapturedAudioFirstFrame();
                ZEGO_SWITCH_THREAD_ING
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_publisher_recv_video_captured_first_frame(enum zego_publish_channel channel, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPublisherCapturedVideoFirstFrame(ZegoPublishChannel(channel));
                ZEGO_SWITCH_THREAD_ING
            }
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_publisher_video_size_changed(int width, int height, enum zego_publish_channel channel, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPublisherVideoSizeChanged(width, height, ZegoPublishChannel(channel));
                ZEGO_SWITCH_THREAD_ING
            }
#endif

            static void zego_on_publisher_update_cdn_url_result(const char *stream_id, zego_error error_code, zego_seq seq, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(stream_id);
                ZEGO_UNUSED_VARIABLE(user_context);
                auto callback = oInternalCallbackCenter->eraseZegoPublisherUpdateCdnUrlCallback(seq);
                if (callback)
                {
                    ZEGO_SWITCH_THREAD_PRE
                    callback(error_code);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            static void zego_on_publisher_relay_cdn_state_update(const char *stream_id, const zego_stream_relay_cdn_info *state_info_list, unsigned int info_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;
                std::vector<ZegoStreamRelayCDNInfo> streamInfoList;

                for (unsigned int i = 0; i < info_count; i++)
                {
                    zego_stream_relay_cdn_info _info = state_info_list[i];
                    ZegoStreamRelayCDNInfo info;
                    info.url = _info.url;
                    info.stateTime = _info.state_time;
                    info.state = ZegoStreamRelayCDNState(_info.cdn_state);
                    info.updateReason = ZegoStreamRelayCDNUpdateReason(_info.update_reason);
                    streamInfoList.push_back(info);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPublisherRelayCDNStateUpdate(streamID, streamInfoList);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_publisher_update_stream_extra_info_result(zego_error error_code, zego_seq seq, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto callback = oInternalCallbackCenter->eraseZegoPublisherSetStreamExtraInfoCallback(seq);
                if (callback)
                {
                    ZEGO_SWITCH_THREAD_PRE
                    callback(error_code);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            // PLAYER CALLBACK
            static void zego_on_player_state_update(const char *stream_id, zego_player_state state, zego_error error_code, const char *extend_data, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;
                std::string extendData = extend_data;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerStateUpdate(streamID, ZegoPlayerState(state), error_code, extendData);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_player_quality_update(const char *stream_id, zego_play_stream_quality quality, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;
                ZegoPlayStreamQuality playQuality = ZegoExpressConvert::I2OPlayQuality(quality);

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerQualityUpdate(streamID, playQuality);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_player_media_event(const char *stream_id, zego_player_media_event media_event, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerMediaEvent(streamID, ZegoPlayerMediaEvent(media_event));
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_player_recv_audio_first_frame(const char *stream_id, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerRecvAudioFirstFrame(streamID);
                ZEGO_SWITCH_THREAD_ING
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_player_recv_video_first_frame(const char *stream_id, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerRecvVideoFirstFrame(streamID);
                ZEGO_SWITCH_THREAD_ING
            }
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_player_render_video_first_frame(const char *stream_id, void *user_context)
            {

                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerRenderVideoFirstFrame(streamID);
                ZEGO_SWITCH_THREAD_ING
            }
#endif


#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_player_video_size_changed(const char *stream_id, int width, int height, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerVideoSizeChanged(streamID, width, height);
                ZEGO_SWITCH_THREAD_ING
            }
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_player_recv_sei(const char *stream_id, const unsigned char *data, unsigned int data_length, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;
                std::vector<unsigned char> seiData;
                seiData.reserve(data_length);
                for (unsigned int i = 0; i < data_length; i++)
                {
                    seiData.push_back(data[i]);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onPlayerRecvSEI(streamID, seiData.data(), ZegoUInt(seiData.size()));
                ZEGO_SWITCH_THREAD_ING
            }
#endif

            // DEVICE CALLBACK
            static void zego_on_audio_device_state_changed(enum zego_update_type update_type, enum zego_audio_device_type device_type, struct zego_device_info _device, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                ZegoDeviceInfo device = ZegoExpressConvert::I2ODeviceInfo(_device);

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onAudioDeviceStateChanged(ZegoUpdateType(update_type), ZegoAudioDeviceType(device_type), device);
                ZEGO_SWITCH_THREAD_ING
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_video_device_state_changed(enum zego_update_type update_type, struct zego_device_info _device, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                ZegoDeviceInfo device = ZegoExpressConvert::I2ODeviceInfo(_device);

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onVideoDeviceStateChanged(ZegoUpdateType(update_type), device);
                ZEGO_SWITCH_THREAD_ING
            }
#endif

            static void zego_on_device_error(zego_error error_code, const char *device_name, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string deviceName = device_name;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onDeviceError(error_code, deviceName);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_remote_mic_state_update(const char *stream_id, zego_remote_device_state state, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRemoteMicStateUpdate(streamID, ZegoRemoteDeviceState(state));
                ZEGO_SWITCH_THREAD_ING
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_remote_camera_state_update(const char *stream_id, zego_remote_device_state state, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string streamID = stream_id;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRemoteCameraStateUpdate(streamID, ZegoRemoteDeviceState(state));
                ZEGO_SWITCH_THREAD_ING
            }
#endif

            static void zego_on_captured_sound_level_update(const zego_sound_level_info *sound_level_info, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                float soundLevel = sound_level_info->sound_level;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onCapturedSoundLevelUpdate(soundLevel);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_remote_sound_level_update(const zego_sound_level_info *sound_level_info_list, unsigned int info_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();

                std::unordered_map<std::string, float> soundLevels;
                for (unsigned int i = 0; i < info_count; i++)
                {
                    zego_sound_level_info sound_level_info = sound_level_info_list[i];
                    std::string streamID = sound_level_info.stream_id;
                    float soundLevel = sound_level_info.sound_level;
                    soundLevels.insert({streamID, soundLevel});
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRemoteSoundLevelUpdate(soundLevels);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_captured_audio_spectrum_update(const zego_audio_spectrum_info *audio_spectrum_info, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                ZegoAudioSpectrum audioSpectrum;
                for (unsigned int i = 0; i < audio_spectrum_info->spectrum_count; i++)
                {
                    audioSpectrum.push_back(audio_spectrum_info->spectrum_list[i]);
                }
                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onCapturedAudioSpectrumUpdate(audioSpectrum);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_remote_audio_spectrum_update(const zego_audio_spectrum_info *audio_spectrum_info_list, unsigned int info_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::unordered_map<std::string, ZegoAudioSpectrum> audioSpectrums;
                for (unsigned int j = 0; j < info_count; j++)
                {
                    zego_audio_spectrum_info frequency_spectrum_info = audio_spectrum_info_list[j];

                    std::string streamID = frequency_spectrum_info.stream_id;
                    ZegoAudioSpectrum frequencySpectrum;
                    for (unsigned int i = 0; i < frequency_spectrum_info.spectrum_count; i++)
                    {
                        frequencySpectrum.push_back(frequency_spectrum_info.spectrum_list[i]);
                    }
                    audioSpectrums.insert({streamID, frequencySpectrum});
                }
                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onRemoteAudioSpectrumUpdate(audioSpectrums);
                ZEGO_SWITCH_THREAD_ING
            }

            // IM CALLBACK
            static void zego_on_im_send_broadcast_message_result(const char *room_id, unsigned long long message_id, zego_error error_code, zego_seq seq, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(room_id);
                ZEGO_UNUSED_VARIABLE(user_context);
                auto callback = oInternalCallbackCenter->eraseZegoIMSendBroadcastMessageCallback(seq);
                if (callback)
                {
                    unsigned long long messageID = message_id;
                    ZEGO_SWITCH_THREAD_PRE
                    callback(error_code, messageID);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            static void zego_on_im_recv_broadcast_message(const char *room_id, const struct zego_broadcast_message_info *message_info_list, unsigned int message_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string roomID = room_id;
                std::vector<ZegoBroadcastMessageInfo> messageInfoList;
                for (unsigned int i = 0; i < message_count; i++)
                {
                    zego_broadcast_message_info message_info = message_info_list[i];
                    ZegoBroadcastMessageInfo messageInfo;
                    messageInfo.messageID = message_info.message_id;
                    messageInfo.sendTime = message_info.send_time;
                    messageInfo.message = message_info.message;
                    messageInfo.fromUser = ZegoExpressConvert::I2OUser(message_info.from_user);
                    messageInfoList.push_back(messageInfo);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onIMRecvBroadcastMessage(roomID, messageInfoList);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_im_send_barrage_message_result(const char *room_id, const char *message_id, zego_error error_code, zego_seq seq, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(room_id);
                ZEGO_UNUSED_VARIABLE(user_context);
                auto callback = oInternalCallbackCenter->eraseZegoIMSendBarrageMessageCallback(seq);
                if (callback)
                {
                    std::string messageID = message_id;
                    ZEGO_SWITCH_THREAD_PRE
                    callback(error_code, messageID);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            static void zego_on_im_recv_barrage_message(const char *room_id, const struct zego_barrage_message_info *message_info_list, unsigned int message_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();

                std::string roomID = room_id;
                std::vector<ZegoBarrageMessageInfo> messageInfoList;
                for (unsigned int i = 0; i < message_count; i++)
                {
                    zego_barrage_message_info message_info = message_info_list[i];
                    ZegoBarrageMessageInfo messageInfo;
                    messageInfo.messageID = message_info.message_id;
                    messageInfo.sendTime = message_info.send_time;
                    messageInfo.message = message_info.message;
                    messageInfo.fromUser = ZegoExpressConvert::I2OUser(message_info.from_user);
                    messageInfoList.push_back(messageInfo);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onIMRecvBarrageMessage(roomID, messageInfoList);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_im_send_custom_command_result(const char *room_id, zego_error error_code, zego_seq seq, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(room_id);
                ZEGO_UNUSED_VARIABLE(user_context);
                auto callback = oInternalCallbackCenter->eraseZegoIMSendCustomCommandCallback(seq);
                if (callback)
                {
                    ZEGO_SWITCH_THREAD_PRE
                    callback(error_code);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            static void zego_on_im_recv_custom_command(const char *room_id, struct zego_user from_user, const char *_content, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();

                std::string roomID = room_id;
                ZegoUser user = ZegoExpressConvert::I2OUser(from_user);
                std::string content = _content;

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onIMRecvCustomCommand(roomID, user, content);
                ZEGO_SWITCH_THREAD_ING
            }

            // MIXER CALLBACK
            static void zego_on_mixer_start_result(zego_error error_code, zego_seq seq, const char *extended_data, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto callback = oInternalCallbackCenter->eraseZegoMixerStartCallback(seq);
                if (callback)
                {
                    int errorCode = error_code;
                    std::string extendData = extended_data;
                    ZEGO_SWITCH_THREAD_PRE
                    callback(errorCode, extendData);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            static void zego_on_mixer_stop_result(zego_error error_code, zego_seq seq, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto callback = oInternalCallbackCenter->eraseZegoMixerStopCallback(seq);
                if (callback)
                {
                    int errorCode = error_code;
                    ZEGO_SWITCH_THREAD_PRE
                    callback(errorCode);
                    ZEGO_SWITCH_THREAD_ING
                }
            }

            static void zego_on_mixer_sound_level_update(const struct zego_mixer_sound_level_info *sound_level_info_list, unsigned int info_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();

                std::unordered_map<unsigned int, float> soundLevels;
                for (unsigned int i = 0; i < info_count; i++)
                {
                    zego_mixer_sound_level_info sound_level_info = sound_level_info_list[i];
                    unsigned int soundLevelID = sound_level_info.sound_level_id;
                    float soundLevel = sound_level_info.sound_level;
                    soundLevels.insert({soundLevelID, soundLevel});
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onMixerSoundLevelUpdate(soundLevels);
                ZEGO_SWITCH_THREAD_ING
            }

            static void zego_on_mixer_relay_cdn_state_update(const char *task_id, const struct zego_stream_relay_cdn_info *cdn_info_list, unsigned int info_count, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoEventHandler();
                std::string taskID = task_id;
                std::vector<ZegoStreamRelayCDNInfo> streamInfoList;

                for (unsigned int i = 0; i < info_count; i++)
                {
                    zego_stream_relay_cdn_info _info = cdn_info_list[i];
                    ZegoStreamRelayCDNInfo info;
                    info.url = _info.url;
                    info.stateTime = _info.state_time;
                    info.state = ZegoStreamRelayCDNState(_info.cdn_state);
                    info.updateReason = ZegoStreamRelayCDNUpdateReason(_info.update_reason);
                    streamInfoList.push_back(info);
                }

                auto weakHandler = std::weak_ptr<IZegoEventHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onMixerRelayCDNStateUpdate(taskID, streamInfoList);
                ZEGO_SWITCH_THREAD_ING
            }

            // MediaPlayer Callback
            static void zego_on_mediaplayer_state_update(zego_mediaplayer_state state, zego_error error_code, zego_mediaplayer_instance_index instance_index, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto mediaPlayer = oInternalCallbackCenter->getZegoExpressMediaPlayerImp(instance_index);
                if (mediaPlayer)
                {
                    mediaPlayer->zego_on_mediaplayer_state_update(state, error_code);
                }
            }

            static void zego_on_mediaplayer_network_event(zego_mediaplayer_network_event event, zego_mediaplayer_instance_index instance_index, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto mediaPlayer = oInternalCallbackCenter->getZegoExpressMediaPlayerImp(instance_index);
                if (mediaPlayer)
                {
                    mediaPlayer->zego_on_mediaplayer_network_event(event);
                }
            }

            static void zego_on_mediaplayer_playing_progress(unsigned long long millisecond, zego_mediaplayer_instance_index instance_index, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto mediaPlayer = oInternalCallbackCenter->getZegoExpressMediaPlayerImp(instance_index);
                if (mediaPlayer)
                {
                    mediaPlayer->zego_on_mediaplayer_playing_progress(millisecond);
                }
            }

            static void zego_on_mediaplayer_seek_to_time_result(zego_seq seq, zego_error error_code, enum zego_mediaplayer_instance_index instance_index, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto mediaPlayer = oInternalCallbackCenter->getZegoExpressMediaPlayerImp(instance_index);
                if (mediaPlayer)
                {
                    mediaPlayer->zego_on_mediaplayer_seek_to_result(seq, error_code);
                }
            }

            static void zego_on_mediaplayer_load_file_result(zego_error error_code, zego_mediaplayer_instance_index instance_index, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto mediaPlayer = oInternalCallbackCenter->getZegoExpressMediaPlayerImp(instance_index);
                if (mediaPlayer)
                {
                    mediaPlayer->zego_on_mediaplayer_load_file_result(error_code);
                }
            }

            static void zego_on_mediaplayer_audio_data(const unsigned char *data, unsigned int data_length, const struct zego_audio_frame_param param, enum zego_mediaplayer_instance_index instance_index, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto mediaPlayer = oInternalCallbackCenter->getZegoExpressMediaPlayerImp(instance_index);
                if (mediaPlayer)
                {
                    mediaPlayer->zego_on_mediaplayer_audio_data(data, data_length, param);
                }
            }

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_mediaplayer_video_data(const unsigned char **data, unsigned int *data_length, const struct zego_video_frame_param param, enum zego_mediaplayer_instance_index instance_index, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto mediaPlayer = oInternalCallbackCenter->getZegoExpressMediaPlayerImp(instance_index);
                if (mediaPlayer)
                {
                    mediaPlayer->zego_on_mediaplayer_video_data(data, data_length, param);
                }
            }
#endif

            // CustomVideoRender
#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_custom_video_render_captured_frame_data(unsigned char **data, unsigned int *data_length, const struct zego_video_frame_param _param, enum zego_video_flip_mode flip_mode, enum zego_publish_channel channel, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoCustomVideoRenderHandler();
                if (handler)
                {
                    auto param = ZegoExpressConvert::I2OVideoFrameParam(_param);
                    ZegoVideoFlipMode flipMode = ZegoVideoFlipMode(flip_mode);
                    handler->onCapturedVideoFrameRawData(data, data_length, param, flipMode, ZegoPublishChannel(channel));
                }
            }
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_custom_video_render_remote_frame_data(const char *stream_id, unsigned char **data, unsigned int *data_length, const struct zego_video_frame_param _param, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoCustomVideoRenderHandler();
                if (handler)
                {
                    std::string streamID = stream_id;
                    auto param = ZegoExpressConvert::I2OVideoFrameParam(_param);
                    handler->onRemoteVideoFrameRawData(data, data_length, param, streamID);
                }
            }
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_custom_video_render_captured_frame_encoded_data(const unsigned char* data, unsigned int data_length, const struct zego_video_encoded_frame_param _param, unsigned long long reference_time_millisecond, enum zego_publish_channel channel, void* user_context)
            {
				ZEGO_UNUSED_VARIABLE(data);
				ZEGO_UNUSED_VARIABLE(data_length);
				ZEGO_UNUSED_VARIABLE(_param);
				ZEGO_UNUSED_VARIABLE(reference_time_millisecond);
				ZEGO_UNUSED_VARIABLE(channel);
                ZEGO_UNUSED_VARIABLE(user_context);
            }
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_custom_video_render_remote_frame_encoded_data(const unsigned char* data, unsigned int data_length, const struct zego_video_encoded_frame_param _param, unsigned long long reference_time_millisecond, const char* stream_id, void* user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoCustomVideoRenderHandler();
                if (handler)
                {
                    std::string streamID = stream_id;
                    auto param = ZegoExpressConvert::I2OVideoEncodedFrameParam(_param);
                    handler->onRemoteVideoFrameEncodedData(data, data_length, param, reference_time_millisecond, streamID);
                }
            }
#endif

            // CustomVideoCapture
#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_custom_video_capture_start(enum zego_publish_channel channel, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoCustomVideoCaptureHandler();
                auto weakHandler = std::weak_ptr<IZegoCustomVideoCaptureHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onStart(ZegoPublishChannel(channel));
                ZEGO_SWITCH_THREAD_ING
            }
#endif

#if ZEGO_EXPRESS_VIDEO_SDK
            static void zego_on_custom_video_capture_stop(enum zego_publish_channel channel, void *user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoCustomVideoCaptureHandler();
                auto weakHandler = std::weak_ptr<IZegoCustomVideoCaptureHandler>(handler);
                ZEGO_SWITCH_THREAD_PRE
                auto handler = weakHandler.lock();
                if (handler)
                    handler->onStop(ZegoPublishChannel(channel));
                ZEGO_SWITCH_THREAD_ING
            }
#endif

            static void zego_on_audio_mixing_copy_data(struct zego_audio_mixing_data *data, void* user_context)
            {
                ZEGO_UNUSED_VARIABLE(user_context);
                auto handler = oInternalCallbackCenter->getIZegoAudioMixingHandler();
                if(handler){
                    ZegoAudioMixingData audioMixingData;
                    audioMixingData.audioData = data->audio_data;
                    audioMixingData.audioDataLength = data->audio_data_length;
                    audioMixingData.param.channel = ZegoAudioChannel(data->param.channel);
                    audioMixingData.param.sampleRate = ZegoAudioSampleRate(data->param.samples_rate);
                    audioMixingData.SEIData = data->sei_data;
                    audioMixingData.SEIDataLength = data->sei_data_length;
                    handler->onAudioMixingCopyData(&audioMixingData);
                    data->audio_data_length = audioMixingData.audioDataLength;
                    data->param.channel = zego_audio_channel(audioMixingData.param.channel);
                    data->param.samples_rate = zego_audio_sample_rate(audioMixingData.param.sampleRate);
                    data->sei_data_length = audioMixingData.SEIDataLength;
                }
            }

			static void zego_on_captured_audio_data(const unsigned char* data, unsigned int data_length, zego_audio_frame_param _param, void* user_context)
			{
				ZEGO_UNUSED_VARIABLE(user_context);
				auto handler = oInternalCallbackCenter->getIZegoAudioDataHandler();
				if (handler) {
					ZegoAudioFrameParam param = ZegoExpressConvert::I2OAudioFrameParam(_param);
					handler->onCapturedAudioData(data, data_length, param);
				}
			}
			
			static void zego_on_remote_audio_data(const unsigned char* data, unsigned int data_length, zego_audio_frame_param _param, void* user_context)
			{
				ZEGO_UNUSED_VARIABLE(user_context);
				auto handler = oInternalCallbackCenter->getIZegoAudioDataHandler();
				if (handler) {
					ZegoAudioFrameParam param = ZegoExpressConvert::I2OAudioFrameParam(_param);
					handler->onRemoteAudioData(data, data_length, param);
				}
			}

			static void zego_on_mixed_audio_data(const unsigned char* data, unsigned int data_length, zego_audio_frame_param _param, void* user_context)
			{
				ZEGO_UNUSED_VARIABLE(user_context);
				auto handler = oInternalCallbackCenter->getIZegoAudioDataHandler();
				if (handler) {
					ZegoAudioFrameParam param = ZegoExpressConvert::I2OAudioFrameParam(_param);
					handler->onMixedAudioData(data, data_length, param);
				}
			}

			static void zego_on_captured_data_record_state_update(zego_data_record_state state, zego_error error_code, zego_data_record_config _config, zego_publish_channel channel, void* user_context)
			{
				ZEGO_UNUSED_VARIABLE(user_context);
				auto handler = oInternalCallbackCenter->getIZegoDataRecordEventHandler();
				auto weakHandler = std::weak_ptr<IZegoDataRecordEventHandler>(handler);
				ZegoDataRecordConfig config = ZegoExpressConvert::I2ODataRecordConfig(_config);
				ZEGO_SWITCH_THREAD_PRE
				auto handler = weakHandler.lock();
				if (handler)
					handler->onCapturedDataRecordStateUpdate(ZegoDataRecordState(state), error_code, config, ZegoPublishChannel(channel));
				ZEGO_SWITCH_THREAD_ING
			}

			static void zego_on_captured_data_record_progress_update(zego_data_record_progress _progress, zego_data_record_config _config, zego_publish_channel channel, void* user_context)
			{
				ZEGO_UNUSED_VARIABLE(user_context);
				auto handler = oInternalCallbackCenter->getIZegoDataRecordEventHandler();
				auto weakHandler = std::weak_ptr<IZegoDataRecordEventHandler>(handler);
				ZegoDataRecordConfig config = ZegoExpressConvert::I2ODataRecordConfig(_config);
				ZegoDataRecordProgress progress;
				progress.duration = _progress.duration;
				progress.currentFileSize = _progress.current_file_size;
				ZEGO_SWITCH_THREAD_PRE
				auto handler = weakHandler.lock();
				if (handler)
					handler->onCapturedDataRecordProgressUpdate(progress, config, ZegoPublishChannel(channel));
				ZEGO_SWITCH_THREAD_ING
			}
        };

    } // namespace EXPRESS
} // namespace ZEGO
