//
// Created by Hansons on 2020/6/21.
//

#ifndef ZEGOEXPRESSENGINE_ZEGO_EXPRESS_CUSTOM_AUDIO_IO_H
#define ZEGOEXPRESSENGINE_ZEGO_EXPRESS_CUSTOM_AUDIO_IO_H

#include "zego-express-defines.h"

/**
 * 抛本地采集的音频数据的回调
 *
 * @param data 音频帧的裸数据, pcm 格式，16 位音频位深
 * @param data_length 数据的长度
 * @param param 音频帧参数
 */
typedef void (*zego_on_captured_audio_data)(
        const unsigned char* data,
        unsigned int data_length,
        struct zego_audio_frame_param param,
        void* user_context
);

/**
 * 抛全部拉流的音频数据的回调
 *
 * @param data 音频帧的裸数据, pcm 格式，16 位音频位深
 * @param data_length 数据的长度
 * @param param 音频帧参数
 */
typedef void (*zego_on_remote_audio_data)(
        const unsigned char* data,
        unsigned int data_length,
        struct zego_audio_frame_param param,
        void* user_context
);

/**
 * 抛本地采集与拉流不放混合的音频数据的回调
 *
 * @param data 音频帧的裸数据, pcm 格式，16 位音频位深
 * @param data_length 数据的长度
 * @param param 音频帧参数
 */
typedef void (*zego_on_mixed_audio_data)(
        const unsigned char* data,
        unsigned int data_length,
        struct zego_audio_frame_param param,
        void* user_context
);

/**
 * 注册将要渲染的纯音频数据外抛的回调
 *
 * @param callback_func 所注册回调的函数指针
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_on_captured_audio_data_callback(
        zego_on_captured_audio_data callback_func,
        void* user_context
);

ZEGOEXP_API void EXP_CALL zego_register_on_remote_audio_data_callback(
        zego_on_remote_audio_data callback_func,
        void* user_context
);

ZEGOEXP_API void EXP_CALL zego_register_on_mixed_audio_data_callback(
        zego_on_mixed_audio_data callback_func,
        void* user_context
);

/**
 * 是否使能将要渲染的纯音频数据外抛的回调
 *
 * @param enable 使能标志
 * @param config 纯音频渲染数据的外抛数据使能配置, 默认值为 44100 HZ，单声道，推流和拉流之后的混音音频
 * @return 错误码
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_audio_data_callback(
        bool enable,
        unsigned int callback_bitmask,
        struct zego_audio_frame_param param
);

/// 开启自定义音频IO功能
/// @param enable 是否开启
/// @param channel 要开启的通道
/// @param config 自定义音频配置
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_custom_audio_io(bool enable, struct zego_custom_audio_config *config, enum zego_publish_channel channel);

/// 发送自定义音频采集 AAC 数据
/// @param data AAC 数据 buffer
/// @param dataLength AAC 数据 buffer 的总长度
/// @param configLength AAC specific config 的长度 (注意: AAC encoded data 的长度 encodedLength = dataLength - configLength)
/// @param referenceTimeMillisecond 时间戳，单位 ms 毫秒
/// @param param 该音频帧的参数
/// @param channel 推流通道
ZEGOEXP_API zego_error EXP_CALL zego_express_send_custom_audio_capture_aac_data(unsigned char *data, unsigned int dataLength, unsigned int configLength, unsigned long long referenceTimeMillisecond, struct zego_audio_frame_param param, enum zego_publish_channel channel);

/// 发送自定义音频采集 PCM 数据
/// @param data PCM 数据 buffer
/// @param dataLength PCM 数据 buffer 的长度
/// @param param 该音频帧的参数
/// @param channel 推流通道
ZEGOEXP_API zego_error EXP_CALL zego_express_send_custom_audio_capture_pcm_data(unsigned char *data, unsigned int dataLength, struct zego_audio_frame_param param, enum zego_publish_channel channel);

/// 拉取自定义音频渲染 PCM 数据
///
/// 建议对接系统框架来定时调用此接口，以驱动音频数据渲染
///
/// @param data 用于存放音频 PCM 数据的内存块，需要用户自行管理此内存块的生命周期
/// @param dataLength 此次需要获取的音频数据的长度 (bufferLength = duration(此音频帧的时长) * sampleRate * channels * 2(位深 16 bit 即 2 Btye))
/// @param param 该音频帧的参数
ZEGOEXP_API zego_error EXP_CALL zego_express_fetch_custom_audio_render_pcm_data(unsigned char *data, unsigned int dataLength, struct zego_audio_frame_param param);

#endif //ZEGOEXPRESSENGINE_ZEGO_EXPRESS_CUSTOM_AUDIO_IO_H
