//
//  zego-express-external-video-io.h
//  ZegoExpressSDK
//
//  Created by 李展鹏@ZEGO on 2019/12/31.
//

#ifndef __ZEGO_EXPRESS_CUSTOM_VIDEO_IO_H__
#define __ZEGO_EXPRESS_CUSTOM_VIDEO_IO_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
* 自定义采集启动通知回调
*
* 收到此回调之后，调用自定义采集相关 API 向 SDK 传递视频数据才会生效
* @param channel 推流通道，标识当前哪条通道启动了自定义采集
* @param user_context 用户上下文
*/
typedef void (*zego_on_custom_video_capture_start)(enum zego_publish_channel channel, void* user_context);

/**
* 自定义采集停止通知回调
*
* 收到此回调之后，调用自定义采集相关 API 向 SDK 传递视频数据将不再生效
* @param channel 推流通道，标识当前哪条通道关闭了自定义采集
* @param user_context 用户上下文
*/
typedef void (*zego_on_custom_video_capture_stop)(enum zego_publish_channel channel, void* user_context);

/**
* 自定义渲染回调采集视频数据
*
* 此回调为高频回调，内部不会切线程。因此不要在此回调中处理耗时任务，否则会导致出帧卡顿
* @param data 视频数据
* @param data_length 视频数据长度
* @param param 视频配置参数
* @param param 视频翻转模式
* @param param 推流通道，标识当前哪条通道启动了自定义渲染
* @param user_context 用户上下文
*/
typedef void (*zego_on_custom_video_render_captured_frame_data)(unsigned char** data, unsigned int* data_length, const struct zego_video_frame_param param, enum zego_video_flip_mode flip_mode, enum zego_publish_channel channel, void* user_context);

/**
* 自定义渲染回调拉流视频数据
*
* 此回调为高频回调，内部不会切线程。因此不要在此回调中处理耗时任务，否则会导致出帧卡顿
* @param stream_id 流ID
* @param data 视频数据
* @param data_length 视频数据长度
* @param param 视频配置参数
* @param user_context 用户上下文
*/
typedef void (*zego_on_custom_video_render_remote_frame_data)(const char* stream_id, unsigned char** data, unsigned int* data_length, const struct zego_video_frame_param param, void* user_context);

/**
* 自定义渲染回调采集视频编码数据
*
* 此回调为高频回调，内部不会切线程。因此不要在此回调中处理耗时任务，否则会导致出帧卡顿
* @param data 视频数据
* @param data_length 视频数据长度
* @param param 视频配置参数
* @param param 视频翻转模式
* @param param 推流通道，标识当前哪条通道启动了自定义渲染
* @param user_context 用户上下文
*/
typedef void(*zego_on_custom_video_render_captured_frame_encoded_data)(const unsigned char* data, unsigned int data_length, const struct zego_video_encoded_frame_param param, unsigned long long reference_time_millisecond, enum zego_publish_channel channel, void* user_context);



/**
* 自定义渲染回调拉流视频编码数据
*
* 此回调为高频回调，内部不会切线程。因此不要在此回调中处理耗时任务，否则会导致出帧卡顿
* @param stream_id 流ID
* @param data 视频数据
* @param data_length 视频数据长度
* @param param 视频配置参数
* @param user_context 用户上下文
*/
typedef void(*zego_on_custom_video_render_remote_frame_encoded_data)(const unsigned char* data, unsigned int data_length, const struct zego_video_encoded_frame_param param, unsigned long long reference_time_millisecond, const char* stream_id, void* user_context);

/**
* 是否开启自定义视频采集
*
* @param enable 是否开启，默认为 false
* @param config 自定义采集配置，传空则使用默认配置（ type 为 zego_video_buffer_type_raw_data）
* @param channel 推流通道，默认为主通道
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_custom_video_capture(bool enable, struct zego_custom_video_capture_config* config, enum zego_publish_channel channel);

/**
* 设置自定义采集画面填充模式
*
* @param mode 填充模式，默认为 zego_view_mode_aspect_fit
* @param channel 推流通道，默认为主通道
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_custom_video_capture_set_fill_mode(enum zego_view_mode mode, enum zego_publish_channel channel);

/**
* 设置自定义采集画面翻转模式
*
* @param mode 翻转模式，默认为 zego_view_mode_aspect_fit
* @param channel 推流通道，默认为主通道
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_custom_video_capture_set_flip_mode(enum zego_video_flip_mode mode, enum zego_publish_channel channel);

/**
* 向 SDK 发送 裸数据 类型的视频数据
*
* @param data 视频数据
* @param data_length 视频数据长度
* @param param 视频配置参数
* @param reference_time 时间戳
* @param reference_time_scale 时间戳单位
* @param channel 推流通道，默认为主通道
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_custom_video_capture_send_memory_data(const unsigned char* data, unsigned int data_length, const struct zego_video_frame_param param, unsigned long long reference_time, unsigned int reference_time_scale, enum zego_publish_channel channel);

/**
* 向 SDK 发送 CVPixelBuffer 类型的视频数据（iOS）
*
* @param buffer 视频数据
* @param reference_time_millisecond 时间戳，单位毫秒
* @param channel 推流通道，默认为主通道
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_custom_video_capture_send_cvpixel_buffer(void* buffer, double reference_time_millisecond, enum zego_publish_channel channel);

/**
* 向 SDK 发送 Texture 类型的视频数据
*
* @param texture_id 纹理ID
* @param width 视频宽度
* @param height 视频高度
* @param reference_time_millisecond 时间戳，单位毫秒
* @param channel 推流通道，默认为主通道
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_custom_video_capture_send_texture_2d_data(int texture_id, int width, int height, double reference_time_millisecond, enum zego_publish_channel channel);

/**
* 向 SDK 发送 编码后的码流 类型的视频数据
*
* @param data 编码后的码流数据
* @param data_length 码流数据长度
* @param param 编码后的码流配置参数
* @param reference_time_millisecond 时间戳，单位毫秒
* @param channel 推流通道，默认为主通道
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_custom_video_capture_send_encoded_frame_data(const unsigned char* data, unsigned int data_length, const struct zego_video_encoded_frame_param param, double reference_time_millisecond, enum zego_publish_channel channel);

/**
* 向 SDK 获取用于编码推流的 SurfaceTexture （Android）
*
* 开发者获取到 SurfaceTexture 之后，往里面写入数据，SDK 将会把数据用于编码推流
* @param channel 推流通道，默认为主通道
* @return (void *), SurfaceTexture 对象
*/
ZEGOEXP_API void* EXP_CALL zego_express_custom_video_capture_get_surface_texture(enum zego_publish_channel channel);

/**
* 是否开启自定义视频渲染
*
* @param enable 是否开启，默认为 false
* @param config 自定义采集配置，传空则使用默认配置（ type 为 zego_video_buffer_type_raw_data, series 为 zego_custom_video_render_series_rgb, is_internal_render 为 false ）
* @return (zego_error), 错误码
*/
ZEGOEXP_API zego_error EXP_CALL zego_express_enable_custom_video_render(bool enable, struct zego_custom_video_render_config* config);

/**
* 注册接收自定义采集开启通知回调
*
* @param callback_func 自定义采集开启回调接口
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_custom_video_capture_start_callback(zego_on_custom_video_capture_start callback_func, void* user_context);

/**
* 注册接收自定义采集停止通知回调
*
* @param callback_func 自定义采集停止回调接口
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_custom_video_capture_stop_callback(zego_on_custom_video_capture_stop callback_func, void* user_context);

/**
* 注册接收自定义渲染抛出采集数据回调
*
* @param callback_func 自定义渲染采集回调接口
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_custom_video_render_captured_frame_data_callback(zego_on_custom_video_render_captured_frame_data callback_func, void* user_context);

/**
* 注册接收自定义渲染抛出拉流数据回调
*
* @param callback_func 自定义渲染流数回调接口
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_custom_video_render_remote_frame_data_callback(zego_on_custom_video_render_remote_frame_data callback_func, void* user_context);

/**
* 注册接收自定义渲染抛出采集编码数据回调
*
* @param callback_func 自定义渲染采集回调接口
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_custom_video_render_captured_frame_encoded_data_callback(zego_on_custom_video_render_captured_frame_encoded_data callback_func, void* user_context);

/**
* 注册接收自定义渲染抛出拉流编码数据回调
*
* @param callback_func 自定义渲染流数回调接口
* @param user_context 用户上下文
*/
ZEGOEXP_API void EXP_CALL zego_register_custom_video_render_remote_frame_encoded_data_callback(zego_on_custom_video_render_remote_frame_encoded_data callback_func, void* user_context);

ZEGO_END_DECLS

#endif /* zego_express_custom_video_io_h */
