#ifndef __ZEGO_EXPRESS_DEFINE_H__
#define __ZEGO_EXPRESS_DEFINE_H__

#ifndef ZEGO_EXPRESS
	#ifdef WIN32
		#ifdef ZEGO_EXPORTS
			#define ZEGO_EXPRESS __declspec(dllexport)
            #define EXP_CALL __cdecl
		#else
			#define ZEGO_EXPRESS __declspec(dllimport)
            #define EXP_CALL __cdecl
		#endif
	#else
		#define ZEGO_EXPRESS __attribute__((visibility("default")))
        #define EXP_CALL 
	#endif
#endif

#ifndef ZEGOEXP_API
    #ifdef ZEGO_C_INTERFACE

        #ifdef WIN32
            #ifdef ZEGO_EXPORTS
                #define ZEGOEXP_API __declspec(dllexport)
                #define EXP_CALL __cdecl
            #else
                #define ZEGOEXP_API __declspec(dllimport)
                #define EXP_CALL __cdecl
            #endif
        #else
            #define ZEGOEXP_API __attribute__((visibility("default")))
            #define EXP_CALL
        #endif

    #else

        #define ZEGOEXP_API //nothing if not build c-interface
        #ifdef WIN32
            #define EXP_CALL __cdecl
        #else
            #define EXP_CALL 
        #endif

    #endif

#endif



#ifdef __cplusplus
#define ZEGO_BEGIN_DECLS    extern "C" {
#define ZEGO_END_DECLS      }
#else
#define ZEGO_BEGIN_DECLS
#define ZEGO_END_DECLS
#endif

#ifndef __cplusplus
#define bool _Bool
#define false 0
#define true 1
#endif

#if defined(__APPLE_OS__) || defined(__APPLE__)
#include "TargetConditionals.h"
#endif

#define ZEGO_EXPRESS_DEFAULT_LOG_SIZE       (5242880)       // 5 * 1024 * 1024 bytes
#define ZEGO_EXPRESS_MAX_LOG_SIZE           (104857600)     // 100 * 1024 * 1024 bytes


/* 定义调试信息级别 */
#define DEBUG_LEVEL_DEBUG		0x01
#define DEBUG_LEVEL_INFO        0x02
#define DEBUG_LEVEL_WARNING     0x03
#define DEBUG_LEVEL_ERROR       0x04


#define DEF_ML(module, level, subno, idx)  ((module<<27)+(level<<24)+(subno<<20)+(idx))

#define ML_SYSTEM_INFO_INIT_SDK DEF_ML(ZEGO_EXPRESS_MODULE_SYSTEM, DEBUG_LEVEL_INFO, 0x01, 0x0);

#define zego_error int
#define zego_seq int
#define zego_instance int

#define ZEGO_EXPRESS_MAX_COMMON_LEN                         (512)
#define ZEGO_EXPRESS_MAX_APPSIGN_LEN                        (64)
#define ZEGO_EXPRESS_MAX_USERID_LEN                         (64)
#define ZEGO_EXPRESS_MAX_USERNAME_LEN                       (256)
#define ZEGO_EXPRESS_MAX_ROOMID_LEN                         (128)
#define ZEGO_EXPRESS_MAX_TOKEN_LEN                          (512)
#define ZEGO_EXPRESS_MAX_MIX_INPUT_COUNT                    (12)
#define ZEGO_EXPRESS_MAX_STREAM_LEN                         (256)
#define ZEGO_EXPRESS_MAX_MIXER_TASK_LEN                     (256)
#define ZEGO_EXPRESS_MAX_EXTRA_INFO_LEN                     (1024)
#define ZEGO_EXPRESS_MAX_DEVICE_ID_LEN                      (256)
#define ZEGO_EXPRESS_MAX_URL_COUNT                          (10)
#define ZEGO_EXPRESS_MAX_URL_LEN                            (1024)
#define ZEGO_EXPRESS_MAX_IMAGE_PATH                         (512)
#define ZEGO_EXPRESS_MAX_MESSAGE_LEN                        (1024)
#define ZEGO_EXPRESS_MAX_CUSTOM_CMD_LEN                     (1024)
#define ZEGO_EXPRESS_MAX_MEDIAPLAYER_INSTANCE_COUNT         (4)
#define ZEGO_EXPRESS_MAX_CAMERA_CAPTURE_INSTANCE_COUNT      (1)
#define ZEGO_EXPRESS_MAX_MEDIA_RECORD_TASK_ID_LEN           (64)


// COMMON
struct zego_user
{
    char userId[ZEGO_EXPRESS_MAX_USERID_LEN];
    char userName[ZEGO_EXPRESS_MAX_USERNAME_LEN];
};

struct zego_stream
{
    struct zego_user user;
    char streamId[ZEGO_EXPRESS_MAX_STREAM_LEN];
    char extraInfo[ZEGO_EXPRESS_MAX_EXTRA_INFO_LEN];
};

// MODULE: ENGINE
enum zego_scenario {
    // 默认场景
    zego_scenario_general,
    // 实时通讯
    zego_scenario_communication,
    // 直播
    zego_scenario_live
};

enum zego_language {
    zego_language_english,
    zego_language_chinese
};

enum zego_engine_state {
    zego_engine_state_start,
    zego_engine_state_stop
};

struct zego_log_config
{
    char log_path[ZEGO_EXPRESS_MAX_COMMON_LEN];
    unsigned long long log_size;
};

enum zego_video_buffer_type
{
    zego_video_buffer_type_unknown = 0,
    zego_video_buffer_type_raw_data = 1,
    zego_video_buffer_type_encoded_data = 2,
    zego_video_buffer_type_gl_texture_2d = 3,
    zego_video_buffer_type_cv_pixel_buffer = 4,
    zego_video_buffer_type_surface_texture = 5
};

enum zego_audio_source_type
{
    zego_audio_source_type_default, // 主通道默认自定义采集，辅通道默认使用与主通道一致的音源
    zego_audio_source_type_custom, // 自定义采集
    zego_audio_source_type_media_player // 媒体播放器，仅辅通道有效
};

enum zego_custom_video_render_series
{
    zego_custom_video_render_series_rgb,
    zego_custom_video_render_series_yuv
};

struct zego_custom_video_capture_config
{
    enum zego_video_buffer_type type;
};

struct zego_custom_video_render_config
{
    enum zego_video_buffer_type type;
    enum zego_custom_video_render_series series;
    bool is_internal_render;
};

struct zego_custom_audio_config
{
    enum zego_audio_source_type source_type;
};

struct zego_engine_config {
    // 日志属性，默认为空（size 为 5M，路径为默认k路径）
    struct zego_log_config* log_config;
    
    // 外部采集主通道选项设置，默认为空（不开启主通道外部采集）
    struct zego_custom_video_capture_config* custom_video_capture_config;
    
    // 外部采集辅助通道选项设置，默认为空（不开启辅助通道外部采集）
    struct zego_custom_video_capture_config* custom_video_capture_aux_config;
    
    // 外部渲染选项设置，默认为空（不开启外部渲染）
    struct zego_custom_video_render_config* custom_video_render_config;
    
    // 隐藏功能开关。多个key-value使用';'隔开，如"config1=A;config2=B"
    char advanced_config[ZEGO_EXPRESS_MAX_COMMON_LEN];
};

// MODULE: ROOM
enum zego_room_state
{
    zego_room_state_disconnected,
    zego_room_state_connecting,
    zego_room_state_connected
};

enum zego_update_type
{
    zego_update_type_add,
    zego_update_type_delete
};

struct zego_room_config
{
    unsigned int max_member_count;
    bool is_user_status_notify;
    // 由开发者业务服务器下发的 token，用以保证安全性，生成规则详见 https://doc.zego.im/CN/565.html
    char thrid_token[ZEGO_EXPRESS_MAX_COMMON_LEN];
};

enum zego_stream_quality
{
    zego_stream_quality_excellent,
    zego_stream_quality_good,
    zego_stream_quality_medium,
    zego_stream_quality_bad,
    zego_stream_quality_die
};

struct zego_rect
{
    int left;
    int top;
    int right;
    int bottom;
};

// MODULE: PUBLISHER
enum zego_publisher_state
{
    zego_publisher_state_no_publish,
    zego_publisher_state_publish_requesting,
    zego_publisher_state_publishing
};

struct zego_publish_stream_quality
{
    enum zego_stream_quality quality;
    
    double video_capture_fps;
    double video_encode_fps;
    double video_send_fps;
    double video_kbps;
    
    double audio_capture_fps;
    double audio_send_fps;
    double audio_kbps;
    
    int rtt;
    double packet_lost_rate;
    
    bool is_hardware_encode;
    
    //int video_width;
    //int video_height;
    
    // 已发送的总字节数，包括音频、视频和SEI等
    double total_send_bytes;
    // 已发送的音频字节数
    double audio_send_bytes;
    // 已发送的视频字节数
    double video_send_bytes;
    
    // 当前 APP 的 CPU 使用率
    //double cpu_app_usage = 0.;
    // 当前系统的 CPU 使用率
    //double cpu_total_usage = 0.;
    
    // 当前 APP 的内存使用率
    //double memory_app_usage = 0.;
    // 当前系统的内存使用率
    //double memory_total_usage = 0.;
    // 当前 APP 的内存使用量，单位MB（windows返回的是app实际占用内存工作集=专用内存工作集+共享内存工作集）
    //double memory_app_used = 0.;
};

enum zego_view_mode
{
    zego_view_mode_aspect_fit,
    zego_view_mode_aspect_fill,
    zego_view_mode_scale_to_fill
};

struct zego_canvas
{
    void* view;
    enum zego_view_mode view_mode;
    // 取值：0xRRGGBB，默认黑色
    int background_color;
};

enum zego_rotation
{
    zego_rotation_0,
    zego_rotation_90,
    zego_rotation_180,
    zego_rotation_270
};

enum zego_orientation
{
    zego_orientation_0,
    zego_orientation_90,
    zego_orientation_180,
    zego_orientation_270
};

enum zego_ios_orientation
{
    zego_ios_orientation_unknown,
    zego_ios_orientation_portrait,
    zego_ios_orientation_portrait_upsidedown,
    zego_ios_orientation_landscape_right,
    zego_ios_orientation_landscape_left
};

enum zego_audio_channel
{
    // 未知
    zego_audio_channel_unknown,
    // 单声道
    zego_audio_channel_mono,
    // 双声道
    zego_audio_channel_stereo
};

enum zego_audio_codec_id
{
    // 根据选择的场景自动适配最优编码模式，一般不建议设置
    zego_audio_codec_id_default,
    zego_audio_codec_id_normal,
    zego_audio_codec_id_normal2,
    zego_audio_codec_id_normal3,
    zego_audio_codec_id_low,
    zego_audio_codec_id_low2,
    zego_audio_codec_id_low3
};

enum zego_video_codec_id
{
    zego_video_codec_id_default,
    zego_video_codec_id_svc,
    zego_video_codec_id_vp8
};

struct zego_audio_config
{
    int bitrate;
    enum zego_audio_channel channel;
    enum zego_audio_codec_id audio_codec_id;
};

struct zego_video_config
{
    int capture_resolution_width;
    int capture_resolution_height;
    
    int encode_resolution_width;
    int encode_resolution_height;
    
    int bitrate;
    int fps;
    
    enum zego_video_codec_id video_codec_id;
};

enum zego_video_mirror_mode
{
    zego_video_mirror_mode_only_preview_mirror,
    zego_video_mirror_mode_both_mirror,
    zego_video_mirror_mode_no_mirror,
    zego_video_mirror_mode_only_publish_mirror
};

enum zego_capture_pipeline_scale_mode
{
    zego_capture_pipeline_scale_mode_pre_scale,
    zego_capture_pipeline_scale_mode_post_scale
};

enum zego_stream_relay_cdn_state
{
    zego_stream_relay_cdn_state_no_relay,
    zego_stream_relay_cdn_state_relay_requesting,
    zego_stream_relay_cdn_state_relaying
};

enum zego_stream_relay_cdn_update_reason
{
    zego_stream_relay_cdn_update_reason_none,
    zego_stream_relay_cdn_update_reason_server_error,
    zego_stream_relay_cdn_update_reason_handshake_failed,
    zego_stream_relay_cdn_update_reason_access_point_error,
    zego_stream_relay_cdn_update_reason_create_stream_failed,
    zego_stream_relay_cdn_update_reason_bad_name,
    zego_stream_relay_cdn_update_reason_cdn_server_disconnected,
    zego_stream_relay_cdn_update_reason_disconnected
};

struct zego_stream_relay_cdn_info
{
    char url[ZEGO_EXPRESS_MAX_URL_LEN];
    enum zego_stream_relay_cdn_state cdn_state;
    enum zego_stream_relay_cdn_update_reason update_reason;
    unsigned int state_time;
};

struct zego_cdn_config
{
    char url[ZEGO_EXPRESS_MAX_URL_LEN];
    char auth_param[ZEGO_EXPRESS_MAX_COMMON_LEN];
};

enum zego_publish_channel
{
    zego_publish_channel_main,
    zego_publish_channel_aux
};

enum zego_traffic_control_property
{
    zego_traffic_control_basic,
    zego_traffic_control_adaptive_fps = 1,
    zego_traffic_control_adaptive_resolution = 1 << 1,
    zego_traffic_control_adaptive_audio_bitrate = 1 << 2
};

enum zego_traffic_control_min_video_bitrate_mode
{
    // 低于设置的最低码率时，停止视频发送
    zego_traffic_control_min_video_bitrate_mode_no_video,
    // 低于设置的最低码率时，视频以极低的频率发送 （不超过2FPS) 
    zego_traffic_control_min_video_bitrate_mode_ultra_low_fps
};

// MODULE: PLAYER
enum zego_player_state
{
    zego_player_state_no_play,
    zego_player_state_play_requesting,
    zego_player_state_playing
};

enum zego_player_video_layer
{
    zego_player_video_layer_auto,            /**< 根据网络状态选择图层  */
    zego_player_video_layer_base,             /**< 指定拉基本层（小分辨率） */
    zego_player_video_layer_extend       /**< 指定拉扩展层（大分辨率)  */
};

struct zego_player_config
{
    // 设置拉流的CDN地址，若设置后，则按照URL拉流而不是按照streamID拉流，此后streamID仅作为SDK内部回调的标识
    struct zego_cdn_config* cdn_config;
    // 设置选用拉流图层，当设置了从CDN拉流后，该参数不生效
    enum zego_player_video_layer video_layer;
};

struct zego_play_stream_quality
{
    enum zego_stream_quality quality;
    
    double video_recv_fps;
    //double video_dejitter_fps;
    double video_decode_fps;
    double video_render_fps;
    double video_kbps;
    
    double audio_recv_fps;
    //double audio_dejitter_fps;
    double audio_decode_fps;
    double audio_render_fps;
    double audio_kbps;
    
    //double audio_break_rate;
    //double video_break_rate;
    
    int rtt;
    double packet_lost_rate;
    // 端到端延迟
    int peer_to_peer_delay;
    // 端到端丢包率
    double peer_to_peer_pkt_lost_rate;
    
    int delay;
    
    bool is_hardware_decode;
    
    //int video_width;
    //int video_height;
    
    // 已接收的总字节数，包括音频、视频和SEI等
    double total_recv_bytes;
    // 已接收的音频字节数
    double audio_recv_bytes;
    // 已接收的视频字节数
    double video_recv_bytes;
    
};

enum zego_player_media_event
{
    // 音频卡顿事件开始
    zego_player_media_event_audio_break_occur,
    // 音频卡顿事件结束
    zego_player_media_event_audio_break_resume,
    // 视频卡顿事件结束
    zego_player_media_event_video_break_occur,
    // 视频卡顿事件结束
    zego_player_media_event_video_break_resume
};

enum zego_remote_device_state
{
    // 设备开启
    zego_remote_device_state_open,
    // 设备关闭：一般性设备错误
    zego_remote_device_state_generic_error,
    // 设备关闭：无效的设备ID
    zego_remote_device_state_invalid_id,
    // 设备关闭：无权限
    zego_remote_device_state_no_authorization,
    // 设备关闭：采集帧率为0
    zego_remote_device_state_zero_fps,
    // 设备关闭：设备被占用
    zego_remote_device_state_in_use_by_other,
    // 设备关闭：设备未插入或被拔出
    zego_remote_device_state_unplugged,
    // 设备关闭：由于系统原因需要重启，如Windows某些机器在设备管理器中禁用了设备，需要重启后才能进行下一次修改，否则重新打开也不生效
    zego_remote_device_state_reboot_required,
    // 设备关闭：系统媒体服务停止，如iOS平台下，当系统检测到当前压力巨大（如播放大量动画），则有可能会将媒体相关服务全部停用
    zego_remote_device_state_system_media_services_lost,
    // 设备关闭：禁用采集，如enableAudioCaptureDevice(false)/enableCamera(false)
    zego_remote_device_state_disable,
    // 设备关闭：远端设备被mute，如enableMic(false)/muteAudioPublish(true)/muteVideoPublish(true)
    zego_remote_device_state_mute,
    // 设备关闭：设备被中断，如电话事件打断等
    zego_remote_device_state_interruption,
    // 设备关闭：用户APP退到后台
    zego_remote_device_state_in_background,
    // 设备关闭：当前前台同时存在多个APP，如iPad应用分屏下，系统会禁止所有应用使用摄像头
    zego_remote_device_state_multi_foreground_app,
    // 设备关闭：系统处于高负载压力下，可能导致设备异常
    zego_remote_device_state_by_system_pressure
};

// MUDOLE: MIXER
enum zego_mixer_input_content_type
{
    zego_mixer_input_content_type_audio,
    zego_mixer_input_content_type_video
};

struct zego_mixer_audio_config
{
    // 混流音频码率，默认48K
    int bitrate;
    
    // 混流编码声道数，默认单声道
    enum zego_audio_channel channel;
    
    // 混流音频编码模式，目前只支持default/normal/normal2/normal3，默认为default
    enum zego_audio_codec_id audio_codec_id;
};

struct zego_mixer_video_config
{
    int resolution_width;
    int resolution_height;
    
    int bitrate;
    int fps;
};

struct zego_mixer_input
{
    enum zego_mixer_input_content_type content_type;
    char stream_id[ZEGO_EXPRESS_MAX_STREAM_LEN];
    struct zego_rect layout;
    unsigned int sound_level_id;
};

struct zego_mixer_output
{
    /**
     * 输出 target 支持流ID和URL形式（URL支持RTMP形式）
     * RTMP URL 格式：rtmp://xxxxxxxx
     */
    char target[ZEGO_EXPRESS_MAX_URL_LEN];
    
    // 混流输出视频配置（混流包含视频流时必填）
    //struct zego_mixer_video_config video_config;
    
    // 混流输出音频配置（可选，默认48K/单声道/default编码格式）
    //struct zego_mixer_audio_config audio_config;
};

struct zego_watermark
{
    /**
     水印图片 ，只支持png、jpg格式。
     */
    char image[ZEGO_EXPRESS_MAX_COMMON_LEN];
    
    struct zego_rect layout;
    
    /**
     *  原点在左上角，x/y/width/height 定义如下：
     *
     *  (x, y)---------------------------
     *  |                                |
     *  |                                |
     *  |                                |
     *  |                                |
     *  -------------------(width, height)
     */
};

struct zego_mixer_sound_level_info
{
    unsigned int sound_level_id;
    float sound_level;
};

struct zego_mixer_task
{
    // 混流任务ID（必须设置）
    char task_id[ZEGO_EXPRESS_MAX_MIXER_TASK_LEN];
    
    // 混流输入流列表（必须设置）
    struct zego_mixer_input* input_list;
    unsigned int input_list_count;
    
    // 混流输出列表（必须设置）
    struct zego_mixer_output* output_list;
    unsigned int output_list_count;

    struct zego_mixer_video_config video_config;

    struct zego_mixer_audio_config audio_config;
    
    // 混流任务设置水印（可选）
    struct zego_watermark* watermark;
    
    // 混流任务设置背景图片（可选）
    char background_image_url[ZEGO_EXPRESS_MAX_URL_LEN];
    
    // 是否开启音浪。（可选）
    // 当混流音浪功能开启后，需要在zego_mixer_input中为每条输入流指定一个唯一的音浪ID。
    // 通过 zego_on_mixer_sound_level_update 中可收到每条已标识流的音浪变化。
    bool enable_sound_level;
};

struct zego_auto_mixer_task
{
    // 自动混流任务ID（必须设置）
    char task_id[ZEGO_EXPRESS_MAX_MIXER_TASK_LEN];
    
    // 房间ID（必须设置）
    char room_id[ZEGO_EXPRESS_MAX_ROOMID_LEN];
    
    // 混流输出列表（必须设置）
    struct zego_mixer_output* output_list;
    unsigned int output_list_count;
    
    // 混流任务音频配置（可选）
//   struct zego_mixer_audio_config audio_config;
    
    // 是否开启声浪
    bool enable_sound_level;
    
};

// MUDOLE: PREPROCESS
struct zego_beautify_option
{
    double polish_step;
    //float polish_factor;
    double whiten_factor;
    double sharpen_factor;
};

enum zego_beautify_feature
{
    zego_beautify_feature_none = 0,
    zego_beautify_feature_polish = 1,
    zego_beautify_feature_whiten = 1 << 1,
    zego_beautify_feature_skin_whiten = 1 << 2,
    zego_beautify_feature_sharpen = 1 << 3
};

enum zego_aec_mode
{
    zego_aec_mode_aggressive,
    zego_aec_mode_medium,
    zego_aec_mode_soft
};

enum zego_ans_mode
{
    zego_ans_mode_soft,
    zego_ans_mode_medium,
    zego_ans_mode_aggressive
};

// MODULE: DEVICE
struct zego_device_info
{
    char device_id[ZEGO_EXPRESS_MAX_COMMON_LEN];
    char device_name[ZEGO_EXPRESS_MAX_COMMON_LEN];
};

struct zego_sound_level_info
{
    char stream_id[ZEGO_EXPRESS_MAX_STREAM_LEN];
    float sound_level;
};

struct zego_audio_spectrum_info
{
    char stream_id[ZEGO_EXPRESS_MAX_STREAM_LEN];
    float* spectrum_list;
    unsigned int spectrum_count;
};

enum zego_audio_device_type
{
    zego_audio_device_type_input,
    zego_audio_device_type_output
};

/* 暂不对外 */
enum zego_audio_capture_channel_type
{
    zego_audio_capture_channel_type_mono,
    zego_audio_capture_channel_type_stereo,
    zego_audio_capture_channel_type_stereo_when_publish_and_play
};

// MODULE: IM
struct zego_broadcast_message_info
{
    char message[ZEGO_EXPRESS_MAX_MESSAGE_LEN];
    unsigned long long message_id;
    unsigned long long send_time;
    struct zego_user from_user;
};

struct zego_barrage_message_info
{
    char message[ZEGO_EXPRESS_MAX_MESSAGE_LEN];
    char message_id[64];
    unsigned long long send_time;
    struct zego_user from_user;
};

enum zego_video_frame_format
{
    zego_video_frame_format_unknown,
    zego_video_frame_format_i420,
    zego_video_frame_format_nv12,
    zego_video_frame_format_nv21,
    zego_video_frame_format_bgra32,
    zego_video_frame_format_rgba32,
    zego_video_frame_format_argb32,
    zego_video_frame_format_abgr32,
    zego_video_frame_format_i422
};

enum zego_video_encoded_frame_format
{
    zego_video_encoded_frame_format_avc_avcc,
    zego_video_encoded_frame_format_avc_annexb
};

enum zego_audio_sample_rate
{
    zego_audio_sample_rate_unknown = 0,
    zego_audio_sample_rate_8k   = 8000,
    zego_audio_sample_rate_16k  = 16000,
    zego_audio_sample_rate_22k  = 22050,
    zego_audio_sample_rate_24k  = 24000,
    zego_audio_sample_rate_32k  = 32000,
    zego_audio_sample_rate_44k  = 44100,
    zego_audio_sample_rate_48k  = 48000
};

struct zego_audio_frame_param
{
    enum zego_audio_channel channel;
    enum zego_audio_sample_rate samples_rate;
};

struct zego_video_frame_param
{
    enum zego_video_frame_format format;
    int strides[4];
    int width;
    int height;
    
    int rotation;
};

struct zego_video_encoded_frame_param
{
    enum zego_video_encoded_frame_format format;
    bool is_key_frame;
    int width;
    int height;
    const unsigned char* sei_data;
    unsigned int sei_data_length;
};

enum zego_video_flip_mode
{
    zego_video_flip_mode_none,
    zego_video_flip_mode_x,
    zego_video_flip_mode_y,
    zego_video_flip_mode_x_y,
};

enum zego_mediaplayer_instance_index
{
    zego_mediaplayer_instance_index_null = -1,
    zego_mediaplayer_instance_index_first = 0,
    zego_mediaplayer_instance_index_second = 1,
    zego_mediaplayer_instance_index_third = 2,
    zego_mediaplayer_instance_index_forth = 3
};

enum zego_mediaplayer_state
{
    zego_mediaplayer_state_no_play,
    zego_mediaplayer_state_playing,
    zego_mediaplayer_state_pausing,
    zego_mediaplayer_state_play_end
};

enum zego_mediaplayer_network_event
{
    zego_mediaplayer_network_event_buffer_begin,
    zego_mediaplayer_network_event_buffer_end
};

struct zego_camera_capturer_params
{
    int capture_resolution_width;
    int capture_resolution_height;
    
    int capture_fps;
};

struct zego_audio_mixing_data
{
    unsigned char* audio_data;
    unsigned int audio_data_length;
    
    struct zego_audio_frame_param param;
    
    unsigned char* sei_data;
    unsigned int sei_data_length;
};

enum zego_volume_type
{
    zego_volume_type_local,
    zego_volume_type_remote
};

/** 录制状态 */
enum zego_data_record_state
{
    /** 未在录制 */
    zego_data_record_state_no_record = 0,

    /** 正在录制中，当调用录制接口之后处于此状态 */
    zego_data_record_state_recording = 1,

    /** 成功录制完成 */
    zego_data_record_state_success = 2

};

/** 文件录制类型 */
enum zego_data_record_type
{
    /** 该字段表示纯音频包默认录制音频，视频包默认录制音视频 */
    zego_data_record_type_defaut = 0,

    /** 只录制音频 */
    zego_data_record_type_only_audio = 1,

    /** 只录制视频 */
    zego_data_record_type_only_video = 2,

    /** 同时录制音频、视频 */
    zego_data_record_type_audio_and_video = 3
};

/**
 * 录制配置
 */
struct zego_data_record_config
{
    /**
     * 录制文件保存路径，绝对路径，需要包含文件名, 文件名需指明后缀，目前只支持 .mp4 或 .flv
     * 以便 SDK 内部会根据文件名后缀存储为指定的文件格式, 若为不支持的文件名后缀即不支持的文件录制格式将报错
     * 若多次录制为同一个路径，[onCapturedMediaRecordStatusUpdate] 回调会有对应错误码
     * 但是不影响正常录制流程，后一次会对存在的录制的文件进行覆盖写入
     */
    char filePath[ZEGO_EXPRESS_MAX_COMMON_LEN];

    /**
     * 录制文件媒体类型
     */
    enum zego_data_record_type record_type;
};

/**
 * 文件录制进度
 */
struct zego_data_record_progress
{
    /** 当前录制时长, 单位为毫秒 */
    unsigned long long duration;

    /** 当前录制文件大小, 单位为字节 */
    unsigned long long current_file_size;
};

struct zego_reverb_param
{
    float damping;
    float dry_wet_ratio;
    float reverberance;
    float room_size;
};

struct zego_reliable_message
{
    char msg_type[10];
    char content[100];
    
    unsigned int latest_seq;
    
    zego_user from_user;
    unsigned long long send_time;
};

struct zego_reliable_message_info
{
    char msg_type[10];
    unsigned int latest_seq;
};

enum zego_platform_language{
    zego_platform_language_c,
    zego_platform_language_oc,
    zego_platform_language_java,
    zego_platform_language_cpp,
    zego_platform_language_dart,
    zego_platform_language_js,
    zego_platform_language_cs
};

#endif /* __ZEGO_EXPRESS_DEFINE_H__ */
