#ifndef __ZEGO_EXPRESS_MEDIA_RECORDER_H__
#define __ZEGO_EXPRESS_MEDIA_RECORDER_H__

#include "zego-express-defines.h"

ZEGO_BEGIN_DECLS

/**
 * 录制到文件的状态更新回调，当录制过程状态变化时触发，具体状态变化参考 [ZegoMediaRecordState]
 *
 * @param task_id 录制任务ID，唯一标识一次录制任务
 * @param state 文件录制状态，开发者应根据此状态来判断文件录制的状态或者进行 UI 的提示等
 * @param error_code 文件录制过程错误码，请参考常见错误码文档 [https://doc-zh.zego.im/zh/308.html]
 * @param user_context 用户上下文
 */
typedef void(*zego_on_captured_media_recorder_state_update)(enum zego_data_record_state state, zego_error error_code, struct zego_data_record_config config, enum zego_publish_channel channel, void* user_context);

/**
 * 录制到文件状态更新回调，回调频率为 1 s
 *
 * @param task_id 录制任务ID，唯一标识一次录制任务
 * @param progress 文件录制过程进度，开发者可以此对用户界面进行 UI 的提示等
 * @param user_context 用户上下文
 */
typedef void(*zego_on_captured_media_record_progress_update)(struct zego_data_record_progress progress, struct zego_data_record_config config, enum zego_publish_channel channel, void* user_context);

/**
 * 注册录制到文件的状态更新回调
 *
 * @param callback_func 录制状态变化回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_recording_captured_data_state_update_callback(zego_on_captured_media_recorder_state_update callback_func, void* user_context);

/**
 * 注册录制到文件的状态进度的回调
 *
 * @param callback_func 录制状态变化回调
 * @param user_context 用户上下文
 */
ZEGOEXP_API void EXP_CALL zego_register_recording_captured_data_progress_update_callback(zego_on_captured_media_record_progress_update callback_func, void* user_context);

/**
 * 开始录制本端音频或音视频，直接将音视频数据录制到文件，录制过程的状态参考 [onCapturedMediaRecordStatusUpdate]
 * 录制过程不应 logoutRoom，若 logoutRoom SDK 内部将主动结束当前录制，并在 [onCapturedMediaRecordStatusUpdate] 抛出对应错误码
 *
 * @param taskID 录制任务ID，唯一标识一次录制任务，长度不超过 64 字节
 * @param config 录制配置对象
 * @param channel 推流通道，若未推流的场景下启动录制或者非推多流的情况下启动录制，可以不关注此参数
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_start_recording_captured_data(struct zego_data_record_config config, enum zego_publish_channel channel);

/**
 * 结束录制
 * 
 * @param taskID 录制任务ID，唯一标识一次录制任务，长度不超过 64 字节
 * @param channel 推流通道，若未推流的场景下启动录制或者非推多流的情况下启动录制，可以不关注此参数
 */
ZEGOEXP_API zego_error EXP_CALL zego_express_stop_recording_captured_data(enum zego_publish_channel channel);

ZEGO_END_DECLS

#endif // __ZEGO_EXPRESS_MEDIA_RECORDER_H__
